#!/usr/bin/perl
#
#
# This module contains a parser for the jobfile format for vdr2iso.
# 		this module is written by ...: geronimo at vdrportal dot de
# 
# COPYRIGHT:	(c) 2005 Creative Commons by-nc-sa
#					some rights reserved. For details on copyright see:
#					http://creativecommons.org/licenses/by-nc-sa
#
# DISCLAIMER:	NO WARANTY AT ALL. Use at your own risk.
#
# vi: ts=3
#		
# DESCRIPTION: see README on a detailled description
#
#
package V2I::jobList;
use Data::Dumper;
use v2icfg;

BEGIN {
	use Exporter   ();
	our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
	$VERSION     = "&vdr2iso-version;";
	@ISA         = qw(Exporter);
	@EXPORT      = qw(&parseJobfile &parseRecording &createJob &checkJob);
	%EXPORT_TAGS = ( );
	@EXPORT_OK   = qw($debug);
}

sub createJob {
	my $job = {};	  

	$job->{'defaultvalues'}{'item'} = {};
	$job->{'disks'} = ();
	$job->{'defaultvalues'}{'item'}{'language'}{'value'}  = $v2icfg::defLang;
	$job->{'defaultvalues'}{'item'}{'aspect'}{'value'}    = $v2icfg::defAspect;
	$job->{'defaultvalues'}{'item'}{'colShadow'}{'value'} = $v2icfg::colShadow;
	$job->{'defaultvalues'}{'item'}{'colBack'}{'value'}   = $v2icfg::colBackground;
	$job->{'defaultvalues'}{'item'}{'colFront'}{'value'}  = $v2icfg::colForeground;
	$job->{'defaultvalues'}{'item'}{'demux'}{'value'}     = $v2icfg::deMuxer;
	$job->{'defaultvalues'}{'item'}{'fntStd'}{'value'}    = $v2icfg::fntStd;
	$job->{'defaultvalues'}{'item'}{'fntStd'}{'size'}     = 26;
	$job->{'defaultvalues'}{'item'}{'fntBig'}{'value'}    = $v2icfg::fntBig;
	$job->{'defaultvalues'}{'item'}{'fntBig'}{'size'}     = 80;
	$job->{'defaultvalues'}{'item'}{'imgBack'}{'value'}   = $v2icfg::bgImage;
	$job->{'defaultvalues'}{'item'}{'cutMarks'}{'value'}  = $v2icfg::fnCutMarks;
	$job->{'defaultvalues'}{'item'}{'chapLen'}{'value'}   = $v2icfg::chapterLength;
	$job->{'defaultvalues'}{'item'}{'header'}{'value'}    = $appText{$appLang}{'defheader'};

	print('createJob: '.Dumper($job)."\n") if $debug;
	return $job;
}

sub addItem {
	my $job = shift;
	my $disk = shift;
	my $section = shift;
	my $tag = shift;
	my %item;
	my $name;

	print("\n".'addItem in section <'.$section.'> with rest['.$tag."]\n") if $debug;
	while($tag =~ /([a-zA-Z_]+)=\"([^"]+)\"/g){
		my $n = $1;
		my $v = $2;
	   if($n =~ /^name$/){
			$name = $v;
			next;
			}	
		$item{$n} = $v;	  
		}
	print("\n".'new item ['.$name.']: '.Dumper(%item)."\n") if $debug;
	$job->{'defaultvalues'}{'item'}{$name} = { %item } if($section =~ /^defaultvalues$/);
	$disk->{'settings'}{'item'}{$name} = { %item } if($section =~ /^settings$/);
}

sub createDisk {
	my $job = shift;
	my $tag = shift;
	my $disk = {};
	
	$disk{'title'} = '';
	$disk{'settings'}{'item'} = {};

	print('createDisk with rest['.$tag."]\n") if $debug;
	while($tag =~ /([a-zA-Z_]+)=\"([^"]+)\"/g){
		$disk->{$1} = $2;	  
		}

	foreach my $key (keys(%{$job->{'defaultvalues'}{'item'}})){
		$disk->{'settings'}{'item'}{$key} = { %{$job->{'defaultvalues'}{'item'}{$key}}};	  
		}
	print("\n".'new disk: '.Dumper($disk)."\n\n") if $debug;
	return $disk;
}

sub addRecording {
	my $disk = shift;
	my $tag = shift;
	my %recording;

	$recording{'lang'} = $disk->{'settings'}{'item'}{'language'}{'value'};
	foreach my $key (qw/demux chapLen/){
		$recording{$key} = $disk->{'settings'}{'item'}{$key}{'value'};	  
		}

	print('addRecording with rest['.$tag."]\n") if $debug;
	while($tag =~ /([a-zA-Z_]+)=\"([^"]+)\"/g){
		$recording{$1} = $2;	  
		}
	print("\n".'new recording: '.Dumper(%recording)."\n") if $debug;
	push(@{$disk->{'recordlist'}{'recording'}}, { %recording });
}

sub parseRecording {
	my $path = shift;
	my @parts = split(/\//, $path);

	my $ts = pop(@parts);
	$ts = pop(@parts) if not "$ts";
	my $name = pop(@parts);
	my $pref = pop(@parts);

	$name =~ s/^%//;
	$name =~ s/_/ /g;
	$pref =~ s/^%//;
	$pref =~ s/_/ /g;

	print('parseRecording: pref="'.$pref."\"\n") if $debug;
	print('                name="'.$name."\"\n") if $debug;
	print('                  ts="'.$ts."\"\n\n") if $debug;
	return ($pref,$name,$ts);
}

sub checkJob {
	my $work = shift;	  
	my $defaultValues = $main::config->{'defaultvalues'}{'item'};
	
	foreach my $job (@{$main::config->{'disks'}}){
		my	$settings = $job->{'settings'}->{'item'};

		$settings->{'imgBack'}   = {%{$defaultValues->{'imgBack'}}}   if not $settings->{'imgBack'}{value};
		$settings->{'colShadow'} = {%{$defaultValues->{'colShadow'}}} if not $settings->{'colShadow'}{value};
		$settings->{'colBack'}   = {%{$defaultValues->{'colBack'}}}   if not $settings->{'colBack'}{value};
		$settings->{'colFront'}  = {%{$defaultValues->{'colFront'}}}  if not $settings->{'colFront'}{value};
		$settings->{'cutMarks'}  = {%{$defaultValues->{'cutMarks'}}}  if not $settings->{'cutMarks'}{value}
																							 and "$defaultValues->{'cutMarks'}{value}";
		$settings->{'language'}  = {%{$defaultValues->{'language'}}}  if not $settings->{'language'}{value};
		$settings->{'aspect'}    = {%{$defaultValues->{'aspect'}}}    if not $settings->{'aspect'}{value};
		$settings->{'fntStd'}    = {%{$defaultValues->{'fntStd'}}}    if not $settings->{'fntStd'}{value};
		$settings->{'fntBig'}    = {%{$defaultValues->{'fntBig'}}}    if not $settings->{'fntBig'}{value};
		$settings->{'header'}    = {%{$defaultValues->{'header'}}}    if not $settings->{'header'}{value};
		$settings->{'demux'}     = {%{$defaultValues->{'demux'}}}     if not $settings->{'demux'}{value};
		$settings->{'chapLen'}   = {%{$defaultValues->{'chapLen'}}}   if not $settings->{'chapLen'}{value};
		for my $i (0 .. $#{$job->{'recordlist'}->{'recording'}}){
			my($prefix,$name,$ts) = parseRecording($job->{'recordlist'}->{'recording'}[$i]->{'path'});
			my $dest = $work.'/VDRSYNC.'.$i;
			my $cmd  = 'mkdir -p \''.$dest.'\'';
			print('command: <|'.$cmd."|>\n") if $main::debug;
			system($cmd);					  

			$job->{'recordlist'}->{'recording'}[$i]->{'path'}   .= '/' if not $job->{'recordlist'}->{'recording'}[$i]->{'path'} =~ /\/$/;  
			$job->{'recordlist'}->{'recording'}[$i]->{'title'}   = $name                              if not $job->{'recordlist'}->{'recording'}[$i]->{'title'};  
			$job->{'recordlist'}->{'recording'}[$i]->{'chapLen'} = $settings->{'chapLen'}->{'value'}  if not $job->{'recordlist'}->{'recording'}[$i]->{'chapLen'};
			$job->{'recordlist'}->{'recording'}[$i]->{'cutMarks'}= $job->{'recordlist'}->{'recording'}[$i]->{'path'}
					  																.$settings->{'cutMarks'}->{'value'} if not $job->{'recordlist'}->{'recording'}[$i]->{'cutMarks'}
																																	  and "$settings->{'cutMarks'}->{'value'}";
			$job->{'recordlist'}->{'recording'}[$i]->{'demux'}   = $settings->{'demux'}->{'value'}    if not $job->{'recordlist'}->{'recording'}[$i]->{'demux'};
			$job->{'recordlist'}->{'recording'}[$i]->{'lang'}    = $settings->{'language'}->{'value'} if not $job->{'recordlist'}->{'recording'}[$i]->{'lang'};
			$job->{'recordlist'}->{'recording'}[$i]->{'ts'}      = $ts;
			$job->{'recordlist'}->{'recording'}[$i]->{'prefix'}  = $prefix;

			if(not "$job->{'recordlist'}->{'recording'}[$i]->{'summary'}"){
				my $summary = $job->{'recordlist'}->{'recording'}[$i]->{'title'};
				my $fpSum;
				if(-s $job->{'recordlist'}->{'recording'}[$i]->{'path'}.'summary.vdr'){
					if(open($fpSum, $job->{'recordlist'}->{'recording'}[$i]->{'path'}.'summary.vdr')){
						$summary = '';	  
						while(my $line = <$fpSum>){
							$summary .= $line;	  
							}
						close($fpSum);
						}
					$job->{'recordlist'}->{'recording'}[$i]->{'summary'} = $summary;
					}
				elsif(-s $job->{'recordlist'}->{'recording'}[$i]->{'path'}.'info.vdr'){
					if(open($fpSum, $job->{'recordlist'}->{'recording'}[$i]->{'path'}.'info.vdr')){
						$summary = '';
						while(my $line = <$fpSum>){
							if($line =~ /^T (.+)$/){
								$job->{'recordlist'}->{'recording'}[$i]->{'title'} = $1;
								}
							elsif($line =~ /^D (.+)$/){
								$job->{'recordlist'}->{'recording'}[$i]->{'summary'} = $1;
								}
							}
						close($fpSum);
						}
					}
				}
			}
		}
	close($fp);
}

sub parseJobfile($) {
	my $fn = shift;
	my $fp;
	my $job;
	my $settings;
	my @section;
	my $defaults;
	my $disk;
	
	open($fp, $fn) or die('Oups, could not open jobfile!');
	my $rawJob = '';
	while(my $line = <$fp>){
		$rawJob .= $line;
		}
	close($fp);

	while($rawJob =~ /<([^>]+)>/g){
		print('tag found: >'.$1."<\n") if $debug;	  
		my $tag = $1;
		my($id,$rest) = split(/ /, $tag, 2);

		print('id of tag: <'.$id.">\n") if $debug;
	   $job = createJob() if($id =~ /^job$/);
		unshift(@section, $id) if($id =~ /^(?:defaultvalues|disks|settings)$/);
		addItem($job, $disk, $section[0], $rest) if($id =~ /^item$/);
		$disk = createDisk($job, $rest) if($id =~ /^disks$/);
		addRecording($disk, $rest) if($id =~ /^recording$/);
		push(@{$job->{'disks'}}, $disk) if($id =~ /^\/disks/);
		}
	return $job;
}

END {}

1;
