#!/usr/bin/perl
#
#
# This script is a counterpart of vdr2iso and is aimed to assist in 
# 		creating the jobfile for processing.
# 		this script is written by ...: geronimo at vdrportal dot de
# 
# COPYRIGHT:	(c) 2005 Creative Commons by-nc-sa
#					some rights reserved. For details on copyright see:
#					http://creativecommons.org/licenses/by-nc-sa
#
# DISCLAIMER:	NO WARANTY AT ALL. Use at your own risk.
#
# vi: ts=3
#		
# INSTALLATION:
# check config file v2icfg.pm for the apropiate settings for your system
# and put that file into /etc/perl.
# 
# DESCRIPTION: see README on a detailled description
#
# HISTORY:
# 15.01.2006 - bugfix on cutmark-handling on recordsplitting
# 
# 14.01.2006 - improved splitting of recordings
# 					improved cache-handling
#					improved preview
#
# 08.01.2006 - improved handling of recordings
#
# 03.01.2006 - new compat-mode
# 
# 01.01.2006 - support for info.vdr
#
# 01.01.2006 - XMLSimple ersetzt
#
# 30.12.2005 - first try				   
# 
use Tk;
use Tk::FileSelect;
use Tk::Dialog;
use Data::Dumper;
use File::Basename;
use GD;
use v2icfg;
use V2I::dvdMenu;
use V2I::jobList;
use V2I::colorSelection;

sub initialize {
	my $fn = shift;	  
	my $rv;
	my $fp;

	$rv = parseJobfile($fn) if(-s $fn);
	$rv = newJob() if(not -s $fn);
	print('parsed jobfile is: '.Dumper($rv)."\n") if $main::debug;

	if(open($fp, $main::fnColors)){
		print('found color-list and gonna process it ...'."\n") if $main::debug;	  
		while(my $line = <$fp>){
			if($line =~ /(\d+) (\d+) (\d+)\s+(.+)/){
				my $red   = $1;
				my $green = $2;
				my $blue  = $3;
				my $name  = uc($4);

				print('save color <'.$name.'> as ('.$red.' / '.$green.' / '.$blue.")\n") if $main::debug;
				$colorTable{byName}{$name} = [($red, $green, $blue)];
				$colorTable{byValue}{hexColor([$red, $green, $blue])} = $name;
				push(@main::colors, $name);
				}
			}
		close($fp);	  
		}
	return $rv;
}

sub createDefaults {
	my $frame = shift;		  
	my $dummy;
	my $f  = $frame->Frame(-relief=>'groove', -bd=>1)->pack(-side=>'right', -fill=>'both', -expand=>0);
	my $rf = $frame->Frame(-relief=>'groove', -bd=>1)->pack(-side=>'right', -fill=>'both', -expand=>1);
	my $mf = $frame->Frame(-relief=>'groove', -bd=>1)->pack(-side=>'right', -fill=>'both', -expand=>1);
	my $sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{imgBack}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my $imgBack = $sf->Label(-text=>$main::config->{'defaultvalues'}{'item'}{'imgBack'}{value}
						 		  , -width=>34
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left');
	$imgBack->bind('<Button-1>', [\&selectFile, 'main::config->{defaultvalues}{item}{imgBack}{value}']);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{fntStd}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my $fntStd = $sf->Label(-text=>$main::config->{'defaultvalues'}{'item'}{'fntStd'}{value}
						 		  , -width=>30
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left');
	my $stdSize = $sf->Label(-text=>$main::config->{'defaultvalues'}{'item'}{'fntStd'}{size}
						 		  , -width=>3
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left');
	$stdSize->bind('<Button-1>', [\&numberInc, \$main::config->{defaultvalues}{item}{fntStd}{size}, 14, 40]);
	$stdSize->bind('<Button-3>', [\&numberDec, \$main::config->{defaultvalues}{item}{fntStd}{size}, 14, 40]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{fntBig}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my $fntBig = $sf->Label(-text=>$main::config->{'defaultvalues'}{'item'}{'fntBig'}{value}
						 		  , -width=>30
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left');
	my $bigSize = $sf->Label(-text=>$main::config->{'defaultvalues'}{'item'}{'fntBig'}{size}
						 		  , -width=>3
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left');
	$fntBig->bind('<Button-1>', [\&selectFile, 'main::config->{defaultvalues}{item}{fntBig}{value}']);
	$bigSize->bind('<Button-1>', [\&numberInc, \$main::config->{defaultvalues}{item}{fntBig}{size}, 14, 150]);
	$bigSize->bind('<Button-3>', [\&numberDec, \$main::config->{defaultvalues}{item}{fntBig}{size}, 14, 150]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{colBack}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	$dummy = hexColor(colorByNameOrValue($main::config->{'defaultvalues'}{'item'}{'colBack'}{value}));
	my $colBack = $sf->Label(-text=>$dummy
						 		  , -background=>$dummy
						 		  , -width=>34
								  , -relief=>'groove')->pack(-side=>'left');
	$colBack->bind('<Button-1>', [\&selectColor, \$config->{'defaultvalues'}{'item'}{'colBack'}{value}]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{colFront}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	$dummy = hexColor(colorByNameOrValue($main::config->{'defaultvalues'}{'item'}{'colFront'}{value}));
	my $colFront = $sf->Label(-text=>$dummy
						 		  , -background=>$dummy
						 		  , -width=>34
								  , -relief=>'groove')->pack(-side=>'left');
	$colFront->bind('<Button-1>', [\&selectColor, \$config->{'defaultvalues'}{'item'}{'colFront'}{value}]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{colShadow}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	$dummy = hexColor(colorByNameOrValue($main::config->{'defaultvalues'}{'item'}{'colShadow'}{value}));
	my $colShadow = $sf->Label(-text=>$dummy
						 			 , -background=>$dummy
						 			 , -width=>34
									 , -relief=>'groove')->pack(-side=>'left');
	$colShadow->bind('<Button-1>', [\&selectColor, \$config->{'defaultvalues'}{'item'}{'colShadow'}{value}]);

	$sf = $mf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{header}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
	my $header = $sf->Entry(-textvariable=>\$config->{'defaultvalues'}{'item'}{'header'}{value}
						 		 , -width=>10)->pack(-side=>'left');

	$sf = $mf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{language}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
	my $language = $sf->Label(-text=>$main::config->{'defaultvalues'}{'item'}{'language'}{value}
						 		   , -width=>3
								   , -anchor=>'e'
								   , -relief=>'groove')->pack(-side=>'left', -padx=>1);
	$language->bind('<Button-1>', [\&numberInc, \$main::config->{defaultvalues}{item}{language}{value}, 0, 9]);
	$language->bind('<Button-3>', [\&numberDec, \$main::config->{defaultvalues}{item}{language}{value}, 0, 9]);

	$sf = $mf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{chapLen}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
	my $chapLen = $sf->Label(-text=>$config->{'defaultvalues'}{'item'}{'chapLen'}{value}
						 		   , -width=>3
								   , -anchor=>'e'
								   , -relief=>'groove')->pack(-side=>'left', -padx=>1);
	$chapLen->bind('<Button-1>', [\&numberInc, \$main::config->{defaultvalues}{item}{chapLen}{value}, 1, 20]);
	$chapLen->bind('<Button-3>', [\&numberDec, \$main::config->{defaultvalues}{item}{chapLen}{value}, 1, 20]);

	$sf = $mf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{demux}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left', -anchor=>'n');
	$sf->Radiobutton(-variable=>\$config->{'defaultvalues'}{'item'}{'demux'}{value}, -text=>'vdrsync.pl', -value=>'vdrsync')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'defaultvalues'}{'item'}{'demux'}{value}, -text=>'ProjectX', -value=>'projectX')->pack(-anchor=>'w');

	$sf = $mf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{cutmarks}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
	my $header = $sf->Entry(-textvariable=>\$config->{'defaultvalues'}{'item'}{'cutMarks'}{value}
						 		 , -width=>15)->pack(-side=>'left');

	$sf = $rf->Frame(-relief=>'groove')->pack(-side=>'left', -anchor=>'nw', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{aspect}.': ', -relief=>'flat', -anchor=>'e')->pack(-side=>'left', -anchor=>'ne');
	$sf->Radiobutton(-variable=>\$config->{'defaultvalues'}{'item'}{'aspect'}{value}, -text=>'1:1', -value=>'1:1')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'defaultvalues'}{'item'}{'aspect'}{value}, -text=>'4:3', -value=>'4:3')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'defaultvalues'}{'item'}{'aspect'}{value}, -text=>'16:9', -value=>'16:9')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'defaultvalues'}{'item'}{'aspect'}{value}, -text=>'2.21:1', -value=>'2.21:1')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'defaultvalues'}{'item'}{'aspect'}{value}, -text=>'letterbox', -value=>'lb')->pack(-anchor=>'w');
}

sub newJob {
	my $rv = createJob();
	$rv->{disks} = ();
	newDisk($rv);
	return $rv;	
}

sub saveJob {
	my $fn = shift;		  
	my $fp;

	if(-s $fn){
		my $cmd = 'mv \''.$fn.'\' \''.$fn.'~\'';	  
		system($cmd);
		}
	if(open($fp, "> $fn")){
		print($fp '<job>'."\n\t".'<defaultvalues>'."\n");	  
		foreach my $key (keys(%{$config->{'defaultvalues'}{'item'}})){
			print($fp "\t\t".'<item name="'.$key.'" ');
			foreach my $skey (keys(%{$config->{'defaultvalues'}{'item'}{$key}})){
				print($fp ' '.$skey.'="'.$config->{'defaultvalues'}{'item'}{$key}{$skey}.'"');	  
				}
			print($fp '/>'."\n");
			}
		print($fp "\t".'</defaultvalues>'."\n");	

		foreach my $disk (@{$config->{'disks'}}){
			my $title = $disk->{'title'};

			$title =~ s/"/'/g;
			print($fp "\t".'<disks title="'.$title.'">'."\n");	  
			print($fp "\t\t".'<settings>'."\n");
			foreach my $key (keys(%{$disk->{'settings'}{'item'}})){
				next if(isDefault(\$disk, $key));	  
				print($fp "\t\t\t".'<item name="'.$key.'" ');
				foreach my $skey (keys(%{$disk->{'settings'}{'item'}{$key}})){
					print($fp ' '.$skey.'="'.$disk->{'settings'}{'item'}{$key}{$skey}.'"');
					}
				print($fp '/>'."\n");	
				}
			print($fp "\t\t".'</settings>'."\n");	
			
			print($fp "\t\t".'<recordlist>'."\n");
			foreach my $rec (@{$disk->{'recordlist'}{'recording'}}){
				print($fp "\t\t\t".'<recording ');
				foreach my $key (keys(%{$rec})){
					next if($rec->{$key} eq $disk->{'settings'}{'item'}{$key}{value});	  
					next if($key eq 'lang' and $rec->{$key} eq $disk->{'settings'}{'item'}{'language'}{value});	  
					my $value = $rec->{$key};

					$value =~ s/"/'/g;
					print($fp ' '.$key.'="'.$value.'"');
					}
				print($fp "/>\n");	
				}
			print($fp "\t\t".'</recordlist>'."\n");
			print($fp "\t".'</disks>'."\n");
			}
		print($fp '</job>'."\n");
		close($fp);			  
		}
}

sub newDisk {
	my $job = shift;
	my $disk = {title=>'disk 0'};
	
	foreach my $key (keys(%{$job->{'defaultvalues'}{'item'}})){
		$disk->{'settings'}{'item'}{$key} = {%{$job->{'defaultvalues'}{'item'}{$key}}};
		}
	$rv->{disks}[0]->{recordlist}->{recording} = ();
	push(@{$job->{disks}}, $disk);
	return $job;
}

sub addDisk {
	print('addDisk - count: '.$#{$config->{'disks'}}."\n") if $main::debug;
	my $disk = {title=>'disk '.($#{$config->{'disks'}}+1)};
	push(@{$config->{'disks'}}, $disk);
	checkJob('/tmp/jobedit');
	$main::lbDisks->insert('end', $disk->{'title'});
}

sub insertDisks {
	my $lb = shift;
	my $jobs = shift;
	my $i = 0;

	foreach my $job (@$jobs){
		$lb->insert('end', $job->{'title'});	  
		}
}

sub setupDiskInfo {	
	my @kids = $main::fRight->children();
	foreach my $w (@kids){
		$w->destroy();	  
		}
	
	checkJob('/tmp/jobedit');

	my $dummy;
	my $frame = $main::fRight;
	my $bf = $frame->Frame(-relief=>'groove', -bd=>1, -label=>$appText{$appLang}{recordings})->pack(-side=>'bottom', -fill=>'both', -expand=>1);
	my $f  = $frame->Frame(-relief=>'groove', -bd=>1)->pack(-side=>'left', -fill=>'both', -expand=>0);
	my $rf = $frame->Frame(-relief=>'groove', -bd=>1)->pack(-side=>'right', -fill=>'both', -expand=>1);
	my $sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{title}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my $imgBack = $sf->Entry(-textvariable=>\$config->{'disks'}[$main::currentDisk]{'title'}
						 		  , -width=>34
								  )->pack(-side=>'left');

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{imgBack}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my $imgBack = $sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'imgBack'}{value}
						 		  , -width=>34
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left', -padx=>1);
	$imgBack->bind('<Button-1>', [\&selectFile, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'imgBack'}{value}]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{fntStd}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my $fntStd = $sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntStd'}{value}
						 		  , -width=>30
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left', -padx=>'1');
	my $stdSize = $sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntStd'}{size}
						 		  , -width=>3
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left');
	$stdSize->bind('<Button-1>', [\&numberInc, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntStd'}{size}, 10, 150]);
	$stdSize->bind('<Button-3>', [\&numberDec, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntStd'}{size}, 10, 150]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{fntBig}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my $fntBig = $sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntBig'}{value}
						 		  , -width=>30
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left', -padx=>'1');
	my $bigSize = $sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntBig'}{size}
						 		  , -width=>3
								  , -anchor=>'e'
								  , -relief=>'groove')->pack(-side=>'left');
	$fntBig->bind('<Button-1>', [\&selectFile, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntBig'}{value}]);
	$bigSize->bind('<Button-1>', [\&numberInc, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntBig'}{size}, 10, 150]);
	$bigSize->bind('<Button-3>', [\&numberDec, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'fntBig'}{size}, 10, 150]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{colBack}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	$dummy = hexColor(colorByNameOrValue($config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'colBack'}{value}));
	my $colBack = $sf->Label(-text=>$dummy
						 		  , -background=>$dummy
						 		  , -width=>34
								  , -relief=>'groove')->pack(-side=>'left', -padx=>1);
	$colBack->bind('<Button-1>', [\&selectColor, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'colBack'}{value}]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{colFront}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	$dummy = hexColor(colorByNameOrValue($config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'colFront'}{value}));
	my $colFront = $sf->Label(-text=>$dummy
						 		  , -background=>$dummy
						 		  , -width=>34
								  , -relief=>'groove')->pack(-side=>'left', -padx=>1);
	$colFront->bind('<Button-1>', [\&selectColor, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'colFront'}{value}]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{colShadow}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	$dummy = hexColor(colorByNameOrValue($main::config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'colShadow'}{value}));
	my $colShadow = $sf->Label(-text=>$dummy
						 			 , -background=>$dummy
						 			 , -width=>34
									 , -relief=>'groove')->pack(-side=>'left', -padx=>1);
	$colShadow->bind('<Button-1>', [\&selectColor, \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'colShadow'}{value}]);

	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{demux}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my @rb = ();
	$rb[0] = $sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'demux'}{value}, -text=>'vdrsync.pl', -value=>'vdrsync')->pack(-side=>'left', -padx=>8);
	$rb[1] = $sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'demux'}{value}, -text=>'ProjectX', -value=>'projectX')->pack(-side=>'left', -padx=>8);
	$rb[0]->bind('<Button-1>', [\&propagateItem, 'demux']); 
	$rb[1]->bind('<Button-1>', [\&propagateItem, 'demux']); 
			  
	$sf = $f->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{cutmarks}.': ', -relief=>'flat', -width=>20, -anchor=>'e')->pack(-side=>'left');
	my $imgBack = $sf->Entry(-textvariable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'cutMarks'}{value}
						 		  , -width=>34
								  )->pack(-side=>'left');

	$sf = $rf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{header}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
	my $header = $sf->Entry(-textvariable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'header'}{value}
						 		 , -width=>10)->pack(-side=>'left');

	$sf = $rf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{aspect}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left', -anchor=>'ne');
	$sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'aspect'}{value}, -text=>'1:1', -value=>'1:1')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'aspect'}{value}, -text=>'4:3', -value=>'4:3')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'aspect'}{value}, -text=>'16:9', -value=>'16:9')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'aspect'}{value}, -text=>'2.21:1', -value=>'2.21:1')->pack(-anchor=>'w');
	$sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'aspect'}{value}, -text=>'letterbox', -value=>'lb')->pack(-anchor=>'w');

	$sf = $rf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{language}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
	my $language = $sf->Label(-text=>$main::config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'language'}{value}
						 		   , -width=>3
								   , -anchor=>'e'
								   , -relief=>'groove')->pack(-side=>'left', -padx=>1);
	$language->bind('<Button-1>', [\&changeAndPropagate, 'inc', 'language', \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'language'}{value}, 0, 9]); 
	$language->bind('<Button-3>', [\&changeAndPropagate, 'dec', 'language', \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'language'}{value}, 0, 9]);

	$sf = $rf->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
	$sf->Label(-text=>$appText{$appLang}{chapLen}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
	my $chapLen = $sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'chapLen'}{value}
						 		   , -width=>3
								   , -anchor=>'e'
								   , -relief=>'groove')->pack(-side=>'left', -padx=>1);
	$chapLen->bind('<Button-1>', [\&changeAndPropagate, 'inc', 'chapLen', \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'chapLen'}{value}, 1, 20]);
	$chapLen->bind('<Button-3>', [\&changeAndPropagate, 'dec', 'chapLen', \$config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{'chapLen'}{value}, 1, 20]);

	my $scroll = $bf->Scrollbar();
	$main::lbRecordings = [$bf->Listbox(-width=>-1), $bf->Listbox(-width=>-1), $bf->Listbox(-width=>-1), $bf->Listbox(-width=>-1), $bf->Listbox(-width=>-1)];
	foreach my $lb (@$main::lbRecordings){
		$lb->configure(-yscrollcommand=>[\&scrollListbox, $scroll, $lb, $main::lbRecordings]);
		$lb->pack(-side=>'left', -expand=>1, -fill=>'y');
		$lb->bind('<Button-1>', \&selectRecording);
		$lb->bind('<Double-Button-1>', \&editRecording);
		}
	$scroll->configure(-command=>[sub {my $listboxes = shift; foreach my $lb (@$listboxes) {$lb->yview(@_)}}, $main::lbRecordings]);
	$scroll->pack(-side=>'right', -fill=>'y');

	foreach my $rec (@{$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}}){
		$main::lbRecordings->[0]->insert('end', $rec->{demux}   ? $rec->{demux}   : $config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{demux}{value});
		$main::lbRecordings->[1]->insert('end', $rec->{lang}    ? $rec->{lang}    : $config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{language}{value});
		$main::lbRecordings->[2]->insert('end', $rec->{chapLen} ? $rec->{chapLen} : $config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{chapLen}{value});
		$main::lbRecordings->[3]->insert('end', $rec->{title}   ? $rec->{title}   : '');
		$main::lbRecordings->[4]->insert('end', $rec->{path});
		}
}

sub changeDiskInfo {
	my $w=shift;
	my $ev=$w->XEvent;
	my $index=$w->nearest($ev->y); 
	
	$main::currentDisk = $index;
	print($index.') selected: '.$w->get($index)."\n") if $main::debug;
	setupDiskInfo();
}

sub localizeNumber {
	my $number = shift;

	$number =~ s/(?<=\d)(?=(\d\d\d)+$)/./g if($appLang eq 'de');
	$number =~ s/(?<=\d)(?=(\d\d\d)+$)/,/g if(not $appLang eq 'de');
	return $number;					 
}

sub localizeTS {
	my $raw = shift;
	my $rv;

	if($raw =~ /^(\d+)-(\d+)-(\d+)\.(\d+)\.(\d+)/){
		if(   $appLang eq 'de') { $rv = $3.'.'.$2.'.'.$1.' '.$4.':'.$5; }
		elsif($appLang eq 'en') { $rv = $2.'/'.$3.'/'.$1.' '.$4.':'.$5; }
		else                    { $rv = $1.'-'.$2.'-'.$3.' '.$4.'.'.$5; }
		}
	else {
		$rv = substr($raw,0,16);			  
		} 
	return $rv;
}

sub propagateItem {
	my $w = shift;
	my $fn = shift;
	my $title = sprintf($appText{$appLang}{mbtitle}, $appText{$appLang}{$fn});
	my $msg = sprintf($appText{$appLang}{qupdate}, $appText{$appLang}{$fn});

	return if($#{$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}} < 1);

	my $dialog = $mw->Dialog(-text =>$msg, -bitmap =>'question', -title =>$title, -default_button =>$appText{$appLang}{'yes'}, -buttons => [$appText{$appLang}{'yes'}, $appText{$appLang}{'no'}]);	
	my $response = $dialog->Show();

	if($response eq $appText{$appLang}{'yes'}){
		foreach my $rec (@{$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}}){
			$rec->{lang} = $config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{language}{value} if($fn eq 'language');	  
			$rec->{$fn} = $config->{'disks'}[$main::currentDisk]{'settings'}{'item'}{$fn}{value} if(not $fn eq 'language');	  
			}
		}
	setupDiskInfo();
}

sub changeAndPropagate {
	my $w = shift;
	my $dir = shift;
	my $fn = shift;
	my $valRef = shift;
	my $min = shift;
	my $max = shift;

	numberInc($w, $valRef, $min, $max) if($dir eq 'inc');
	numberDec($w, $valRef, $min, $max) if($dir eq 'dec');
	propagateItem($w, $fn);
}

sub scrollListbox {
	my($sb, $scrolled, $lbs, @args) = @_;
	
	$sb->set(@args);
	my($top, $bottom) = $scrolled->yview();

	foreach my $list (@$lbs){
		$list->yviewMoveto($top);
		}
}

sub numberInc {
	my $w = shift;
	my $name = shift;
	my $min = shift;
	my $max = shift;
	print('numberinc ...'."\n") if $main::debug;
	${$name}++ if ${$name} < $max;
	$w->configure(-text=>$$name);
}

sub numberDec {
	my $w = shift;
	my $name = shift;
	my $min = shift;
	my $max = shift;
	print('numberdec ...'."\n") if $main::debug;
	${$name}-- if ${$name} > $min;
	$w->configure(-text=>$$name);
}

sub selectFile {
	my $w = shift;
	my $vn = shift;
	my $dir = shift;

	print('selectFile() - dirname ['.$$dir."]\n");
	
	if($main::fileSelection{win} and Exists($main::fileSelection{win})){
		$fileSelection{win}->deiconify();
		$fileSelection{win}->raise();
		}
	else{
		${$vn} = '.' if not "${$vn}";	  
		$startDir = dirname(${$vn}) if not "$dir";
		$startDir = $$dir if "$$dir";
		$main::fileSelection{win} = $mw->FileSelect(-directory=>$startDir);
		$main::fileSelection{var}{org} = ${$vn};

		my $path = $main::fileSelection{win}->Show;
		$main::fileSelection{win} = 0;
		${$vn} = $path if "$path";
		$w->configure(-text=>${$vn});
		}
}

sub find {
	my $path = shift;
	my $what = shift;
	my $when = shift;
	my $dp;
	my @entries = ();

   opendir($dp, $path) or die "can't opendir $path: $!";
	while(my $entry = readdir($dp)){
		next if($entry =~ /^\.\.?$/);	  
		if(-d $path.'/'.$entry) {
			my $subEntries = find($path.'/'.$entry, $what, $when);	  
			foreach (@$subEntries){
				push(@entries, $_);	  
				}
			}
	   if($entry eq $what){
			my @st = stat($path.'/'.$entry);	  
			next if($when and $st[9] < $when);
			push(@entries, $path.'/'.$entry);		  
			}
		}
   closedir($dp);
	return \@entries;
}

sub preview {
	my $work = '/tmp/jobedit';	  
	checkJob($work);
	createMenus($config->{'disks'}[$main::currentDisk], $work);
	dvdMenuPreview($work.'/menu-main.png');
}

sub xPreview {
	my $edit = shift;
	my $work = '/tmp/jobedit';	  

	$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]->{summary} = $edit->get('1.0', 'end');
	checkJob($work);
	createMenus($config->{'disks'}[$main::currentDisk], $work);
	dvdMenuPreview($work.'/VDRSYNC.'.$main::currentRecording.'/menu-bg.png');
}

sub selectRecording {
	my $lb = shift;	  
	my @selection = $lb->curselection();

	$main::currentRecording = $selection[0];
	my $number = sprintf("%03d", $main::currentRecording);
	print('selectRecording: - current Recording #'.$number."\n") if $main::debug;
}

sub cloneRecording {
	my %newRec;
	
	foreach my $key (keys(%{$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]})){
		print('cloneRecording: ['.$key.'] => { '.$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{$key}." }\n") if $main::debug;
		$newRec{$key} = $config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{$key};
		}
	push(@{$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}}, {%newRec});
	setupDiskInfo();
}

sub editRecording {
	if($main::editRecordingWindow and Exists($main::editRecordingWindow)){
		$main::editRecordingWindow->deiconify();
		$main::editRecordingWindow->raise();
		}
	else{
		my $lb = shift;
		my @selection = $lb->curselection();
		my $number = sprintf("%03d", $selection[0]);
		my $w = $mw->Toplevel();	  

		$w->title($appText{$appLang}{recording});
		$main::editRecordingWindow = $w;

		my $bf = $w->Frame(-relief=>'groove')->pack(-side=>'bottom');
		$bf->Button(-text=>$appText{$appLang}{done}, -command=>sub {$w->withdraw; $main::editRecordingWindow=0; setupDiskInfo(); })->pack(-side=>'left');
		$bf->Button(-text=>$appText{$appLang}{clone}, -command=>\&cloneRecording)->pack(-side=>'left', -padx=>10);
		$bf->Button(-text=>$appText{$appLang}{projectx}, -command=>\&startProjectX)->pack(-side=>'left');

		my $sf = $w->Frame(-relief=>'groove')->pack(-side=>'top', -anchor=>'w', -fill=>'x');
		$sf->Label(-text=>$appText{$appLang}{path}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
		$sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'path'}
					, -relief=>'flat')->pack(-side=>'left');
	
		my $edit = $w->Scrolled('Text', -relief=>'groove', -scrollbars=>'osoe', -width=>47, -height=>5)->pack(-side=>'right');
		$edit->configure(-wrap=>'word', -setgrid=>1);
		$edit->insert('end', $config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]->{summary});
		$edit->configure(-state=>'disabled');

		$sf = $w->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
		$sf->Label(-text=>$appText{$appLang}{title}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
		$sf->Entry(-textvariable=>\$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'title'}
			 		 , -width=>30)->pack(-side=>'left');

		$sf = $w->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
		$sf->Label(-text=>$appText{$appLang}{demux}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
		$sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'demux'}
							 , -text=>'vdrsync.pl', -value=>'vdrsync')->pack(-side=>'left');
		$sf->Radiobutton(-variable=>\$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'demux'}
							 , -text=>'ProjectX', -value=>'projectX')->pack(-side=>'left');
			  
		$sf = $w->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
		$sf->Label(-text=>$appText{$appLang}{language}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
		my $language = $sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'lang'}
							 		   , -width=>3
									   , -anchor=>'e'
									   , -relief=>'groove')->pack(-side=>'left');
		$language->bind('<Button-1>', [\&numberInc, \$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'lang'}
							 , 0, 9]);
		$language->bind('<Button-3>', [\&numberDec, \$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'lang'}
							 , 0, 9]);
		$sf->Label(-text=>$appText{$appLang}{chapLen}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
		my $chapLen = $sf->Label(-text=>$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'chapLen'}
							 		   , -width=>3
									   , -anchor=>'e'
									   , -relief=>'groove')->pack(-side=>'left');
		$chapLen->bind('<Button-1>', [\&numberInc, \$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'chapLen'}
							 , 1, 20]);
		$chapLen->bind('<Button-3>', [\&numberDec, \$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'chapLen'}
							 , 1, 20]);		

		$sf = $w->Frame(-relief=>'groove')->pack(-anchor=>'w', -fill=>'x');
		$sf->Label(-text=>$appText{$appLang}{cutmarks}.': ', -relief=>'flat', -width=>15, -anchor=>'e')->pack(-side=>'left');
		my $cutmarks = $sf->Label(-text=>$main::config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'cutMarks'}
							 			 , -width=>34
										 , -anchor=>'e'
										 , -relief=>'groove')->pack(-side=>'left');
		$cutmarks->bind('<Button-1>', [\&selectFile, \$main::config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'cutMarks'}, \$main::config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]{'path'}]);
		}
}

sub changeRelatedRecordings {
	my $listboxes = shift;		  
	my @selection = $listboxes->{name}->curselection();

	$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'} = ();
	foreach my $index (@selection){
		my $rec = {path=>$listboxes->{path}->get($index)};	  
		push(@{$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}}, $rec);
		}
	setupDiskInfo();
	$main::recordingListWindow->withdraw();
	$main::recordingListWindow = 0;
}

sub scanRecording {
	my $record = shift;
	my $cmd = 'vdrsync.pl -i \''.$record->{path}.'\'';
	my $fp;
	my (@streams, $streamType, $streamID, $codec, $sampleFreq, $bitRate, $eBitRate, $streamType);
	my ($aspect, $width, $height, $mode, $speed, $frameBytes, $frameTicks, $size);
	my $movTotal = 1;
	my	$movCSize = 1;

	if(open($fp, "$cmd |")){
		my $hasAC3 = 0;	  
		my $i = 0;
		my $startSeen = 0;

		print(stderr '.');
		print('scan: '.$record->{path}."\n") if $main::debug;
		while(my $line = <$fp>){
			$startSeen = 1 if($line =~ /^READING INDEX FILE, please be patient$/);
			next if not $startSeen;
		   if($line =~ /^movie_size\s+:\s+(\d+)/)     { $movTotal = $1; }
		   if($line =~ /^movie_size_cut\s+:\s+(\d+)/) { $movCSize = $1; }
			if($line =~ /(audio|video)\s+stream\s+([^\s]+)\s+info(\s+\(([^)]+)\))?:/){
				print("found $1-stream with ID: #$2 of type $3\n") if $main::Debug;  
				$streamType = $1;
				$streamID = $2;
				$codec = $3;
				$hasAC3 = 1 if($codec =~ /AC3/);
				if($streamType =~ /audio/){
					while(my $line = <$fp>){
						$sampleFreq = $1 if($line =~ /Sample frequency:\s+(\d+)/);
						$bitRate    = $1 if($line =~ /Bitrate:\s+(\d+)/);
						$mode       = $1 if($line =~ /Mode:\s+([^\s]+)/);
						$frameBytes = $1 if($line =~ /Frame length \(bytes\)\s+(\d+)/);
						$frameTicks = $1 if($line =~ /Frame length \(ticks\)\s+(\d+)/);
						$size       = $1 if($line =~ /App. Size \(bytes\)\s+(\d+)/);
						last if($line =~ /^\s*$/);
						}
					$streams[$i]{type}     = $streamType;
					$streams[$i]{id}       = $streamID;
					$streams[$i]{codec}    = $codec;
					$streams[$i]{sample}   = $sampleFreq;
					$streams[$i]{bitrate}  = $bitRate;
					$streams[$i]{eBitrate} = $bitRate;
					$streams[$i]{mode}     = $mode;
					$streams[$i]{aspect}   = 0;
					$streams[$i]{width}    = 0;
					$streams[$i]{height}   = 0;
					$streams[$i]{speed}    = 0;
					$streams[$i]{bFrLen}   = $frameBytes;
					$streams[$i]{tFrLen}   = $frameTicks;
					$streams[$i]{size}     = $size;
					}
				elsif($streamType =~ /video/){
					while(my $line = <$fp>){
						$frameTicks = $1 if($line =~ /Frame length \(ticks\)\s+(\d+)/);
						$aspect     = $1 if($line =~ /Aspect ratio\s+([\d\:]+)/);
						$width      = $1 if($line =~ /Horizontal size\s+(\d+)/);
						$height     = $1 if($line =~ /Vertical size\s+(\d+)/);
						$speed      = $1 if($line =~ /Frames per Second\s+(\d+)/);
						$bitRate    = $1 if($line =~ /Bitrate\s+(\d+)/);
						$eBitRate   = $1 if($line =~ /Bitrate \(estm\)\s+(\d+)/);
						$size       = $1 if($line =~ /App. Size \(bytes\)\s+(\d+)/);
						last if($line =~ /^\s*$/);
						}
					$streams[$i]{type}     = $streamType;
					$streams[$i]{id}       = $streamID;
					$streams[$i]{codec}    = $codec;
					$streams[$i]{sample}   = 0;
					$streams[$i]{bitrate}  = $bitRate;
					$streams[$i]{eBitrate} = $eBitRate;
					$streams[$i]{mode}     = $streamType;
					$streams[$i]{aspect}   = $aspect;
					$streams[$i]{width}    = $width;
					$streams[$i]{height}   = $height;
					$streams[$i]{speed}    = $speed;
					$streams[$i]{bFrLen}   = 0;
					$streams[$i]{tFrLen}   = $frameTicks;
					$streams[$i]{size}     = $size;
					}
				$i++;
				}
			}
		close($fp);
		$record->{raw_size} = $movTotal;
		$record->{cut_size} = $movCSize;
		$record->{info}     = [@streams];
		$record->{hasAC3}   = $hasAC3;
		}
}

sub readRecordings {
	my $fp;	  
	my %recordings;
	my $tm = 0;

	if(-s $main::cacheFn){
		my @st = stat($main::cacheFn);

		$tm = $st[9];
		if(open($fp, $main::cacheFn)){
			while(my $line = <$fp>){
				$line =~ s/\n$//;
					  
				my $recInfo = {};
				my @parts = split(/;/, $line);

				$recInfo->{prefix}   = $parts[0];
				$recInfo->{name}     = $parts[1];
				$recInfo->{aspect}   = $parts[2];
				$recInfo->{audio}    = $parts[3];
				$recInfo->{size}     = $parts[4];
				$recInfo->{cut_size} = $parts[5];
				$recInfo->{raw_size} = $parts[6];
				$recInfo->{ts}       = $parts[7];
				$recInfo->{path}     = $parts[8];
				$recordings{$parts[8]} = $recInfo;
				}
			close($fp);	  
			}
		}
	else{
		print("\n".$appText{$appLang}{nocache}."\n\t".$appText{$appLang}{gonnascan}."\n\n");	  
		}
	my $newRecordings = find($imp, 'index.vdr', $tm);

	foreach my $recPath (@$newRecordings){
		print('new recording ...: '.$recPath."\n");
		}
		
	if(@{$newRecordings}){
		foreach my $recPath (@$newRecordings){
			$recPath =~ s/index.vdr$//;

			my $elem = {path=>$recPath};
			my($pref,$rec,$ts) = parseRecording($recPath);

			$elem->{prefix} = $pref;
			$elem->{name} = $rec;
			$elem->{ts} = $ts;

			scanRecording($elem);

			my $size = 0;
			my $aspect = 'unknown';
			foreach my $i (0 .. $#{$elem->{info}}){
				if($elem->{info}[$i]{type} =~ /video/){
					print("Vidio-stream has size ".$elem->{info}[$i]{size}."\n") if $main::Debug;	  
					$size += $elem->{info}[$i]{size}; 
					$aspect = $elem->{info}[$i]{aspect};
					}	  
				if($elem->{hasAC3}){
				 	if($elem->{info}[$i]{codec} =~ /AC3/){
						print("found AC3-stream with size ".$elem->{info}[$i]{size}."\n") if $main::Debug;	  
						$size += $elem->{info}[$i]{size};
						}
					}
				elsif($elem->{info}[$i]{mode} =~ /stereo/) {
					print("found standard-stream with size ".$elem->{info}[$i]{size}."\n") if $main::Debug;	  
					$size += $elem->{info}[$i]{size};
					}
				}
			my $factor = $elem->{cut_size} / $elem->{raw_size};
			$size *= $factor * 1.0285 if $elem->{hasAC3};
			$size *= $factor * 1.026  if not $elem->{hasAC3};
			$elem->{aspect} = $aspect;
			$elem->{size}   = int($size);
			$elem->{audio}  = $elem->{hasAC3}?"ac3":"st";
			$recordings{$elem->{path}} = $elem;
			}
		open($fp, '> '.$main::cacheFn) or die('could not create cache!'."\n");
		foreach my $recPath (keys(%recordings)){
			my $entry = $recordings{$recPath};
			printf($fp "%s;%s;%s;%s;%s;%s;%s;%s;%s\n"
					, $entry->{prefix}
					, $entry->{name}
					, $entry->{aspect}
					, $entry->{audio}
					, $entry->{size}
					, $entry->{cut_size}
					, $entry->{raw_size}
					, $entry->{ts}
					, $entry->{path});
			}
		close($fp);
		}
	print("\n");
	return \%recordings;
}
		
sub addRecording {
	if($main::recordingListWindow and Exists($main::recordingListWindow)){
		$main::recordingListWindow->deiconify();
		$main::recordingListWindow->raise();
		}
	else{
		my $w = $mw->Toplevel();	  

		$w->title($appText{$appLang}{recordings});
		$main::recordingListWindow = $w;

		my $sf = $w->Frame(-relief=>'groove')->pack(-side=>'bottom');
		my $st = $w->Frame(-relief=>'groove')->pack(-side=>'top', -expand=>1, -fill=>'x');
		my $scroll = $w->Scrollbar();
		my $listBoxes = {aspect=>$w->Listbox(-width=>-1, -height=>99)
							 ,audio=>$w->Listbox(-width=>-1, -height=>99)
							 ,size=>$w->Listbox(-width=>-1, -height=>99)
							 ,ts=>$w->Listbox(-width=>-1, -height=>99)
				  			 ,name=>$w->Listbox(-width=>-1, -height=>99, -selectmode=>'extended')
							 ,prefix=>$w->Listbox(-width=>-1, -height=>99)
							 ,path=>$w->Listbox(-width=>1, -height=>99)};
		my $arListboxes = [$listBoxes->{aspect}
								 ,$listBoxes->{audio}
								 ,$listBoxes->{size}
								 ,$listBoxes->{ts}
								 ,$listBoxes->{name}
								 ,$listBoxes->{prefix}
								 ,$listBoxes->{path}
								 ];
		foreach (qw/aspect audio size ts name prefix path/){
			my $list = $listBoxes->{$_};	  
			$list->configure(-yscrollcommand=>[\&scrollListbox, $scroll, $list, $arListboxes]);
			$list->pack(-side=>'left');
			}
		$scroll->configure(-command=>[sub {my $listboxes=shift; foreach my $lb (@$listboxes){$lb->yview(@_)}}, $arListboxes]);
		$scroll->pack(-side=>'right', -fill=>'y');

		my $edit = $st->Scrolled('Text', -padx=>5, -relief=>'flat', -scrollbars=>'osoe', -height=>6)->pack(-expand=>1, -fill=>'x');
		$edit->insert('end', $appText{$appLang}{hintaddrec});
		$edit->configure(-wrap=>'word', -state=>'disabled');

		my $allRecordings = readRecordings();
		@recordings = sort { uc($a->{prefix}) cmp uc($b->{prefix}) || uc($a->{name}) cmp uc($b->{name}) || $a->{ts} cmp $b->{ts}} values(%$allRecordings);
		my $i = 0;
		foreach my $rec (@recordings){
			$listBoxes->{aspect}->insert('end', $rec->{aspect});
			$listBoxes->{audio}->insert('end', $rec->{audio});
			$listBoxes->{size}->insert('end', sprintf("%14s",localizeNumber($rec->{size})));
			$listBoxes->{ts}->insert('end', $rec->{ts});
			$listBoxes->{name}->insert('end', $rec->{name});
			$listBoxes->{prefix}->insert('end', $rec->{prefix});
			$listBoxes->{path}->insert('end', $rec->{path});
				  
			foreach my $elem (@{$config->{'disks'}[$main::currentDisk]{'recordlist'}->{'recording'}}){
				if($elem->{'path'} eq $rec->{'path'}){
					print('recording matched at #'.$i.' for <'.$elem->{'path'}.">\n") if $main::debug;	  
					$listBoxes->{name}->selectionSet($i);
					}
				else{
					print('no match of <'.$rec->{'path'}.'> and <'.$elem->{'path'}.">\n") if $main::debug;	  
					}
				}
			$i++;
			}
		$sf->Button(-text=>$appText{$appLang}{ok}, -command=>[\&changeRelatedRecordings, $listBoxes])->pack(-side=>'left');
		$sf->Button(-text=>$appText{$appLang}{rescan}, -command=>[\&rescanRecording, $listBoxes])->pack(-side=>'right');
		$sf->Button(-text=>$appText{$appLang}{split}, -command=>[\&splitRecording, $listBoxes])->pack(-side=>'right');
		$sf->Button(-text=>$appText{$appLang}{projectx}, -command=>[\&startProjectX, $listBoxes])->pack(-side=>'right', -padx=>10);
		$sf->Button(-text=>$appText{$appLang}{cancel}, -command=>sub {$w->withdraw; $main::recordingListWindow=0;})->pack(-side=>'right');
		}
}

sub rescanRecording {
	my $cmd = 'rm -f '.$main::cacheFn;
	system($cmd);
}

sub splitRecording {
	my $listboxes = shift;		  

	if($main::splitRecordingWindow and Exists($main::splitRecordingWindow)){
		$main::splitRecordingWindow->deiconify();
		$main::splitRecordingWindow->raise();
		}
	else{
		my @selection = ();	  
		foreach (qw/aspect audio size ts name prefix path/){
			@selection = $listboxes->{$_}->curselection();
			last if(scalar(@selection));
			}
		return if(not scalar(@selection));	  
		
		my $path = $listboxes->{path}->get($selection[0]);
		my $size = $listboxes->{size}->get($selection[0]);
		my $w = $mw->Toplevel();
		my $f = $w->Frame()->pack(-side=>'right', -expand=>1, -fill=>'both');

		$w->title($appText{$appLang}{split});
		$main::splitRecordingWindow = $w;
		$w->Label(-text=>$path)->pack(-side=>'top');

		my $hint = $w->Scrolled('Text', -relief=>'flat', -scrollbars=>'osoe', -width=>70, -height=>6)->pack(-side=>'top', -expand=>1, -fill=>'x');
		$hint->configure(-wrap=>'word');
		$hint->insert('end', $appText{$appLang}{hintsplit});

		my $sf = $w->Frame(-relief=>'groove')->pack(-side=>'bottom');
		my $lf = $w->Frame()->pack(-side=>'left', -expand=>1, -fill=>'both');

		my $totalSize = recLength($path);
		my $nMarks = readMarks($path);
		my $dvdSize = 4700000000;
		my $splitAt = $dvdSize;
		$splitAt = $nMarks->{base}[0] + $dvdSize if(($nMarks->{base}[0] + $dvdSize) < $nMarks->{base}[$#{$nMarks->{base}}]);
		$splitAt = ($nMarks->{base}[0] + $nMarks->{base}[$#{$nMarks->{base}}]) /2 if(($nMarks->{base}[0] + $dvdSize) >= $nMarks->{base}[$#{$nMarks->{base}}]);
		$lf->Label(-text=>$appText{$appLang}{totalsize}.localizeNumber($totalSize), -anchor=>'e')->pack(-anchor=>'w', -expand=>1, -fill=>'x', -padx=>20);
		$lf->Label(-text=>$appText{$appLang}{estimated}.localizeNumber($size), -anchor=>'e')->pack(-anchor=>'w', -expand=>1, -fill=>'x', -padx=>20);
		foreach my $mark (@{$nMarks->{base}}){
			$lf->Label(-text=>$appText{$appLang}{'cutmarkat'}.': '.sprintf("%13s", localizeNumber($mark)), -anchor=>'e')->pack(-anchor=>'w', -expand=>1, -fill=>'x', -padx=>20);	  
			}
		my $lblSplit = $lf->Label(-text=>'split at: '.sprintf("%13s", localizeNumber($splitAt)),  -anchor=>'e');
		$lblSplit->pack(-anchor=>'w', -expand=>1, -fill=>'x', -padx=>20);	  
		$f->Scale(-from=>0, -to=>$totalSize, -command=>[\&showSplitValue, \$lblSplit], -showvalue=>0, -length=>300, -variable=>\$splitAt)->pack(-side=>'right', -expand=>1, -fill=>'y');

		$sf->Button(-text=>$appText{$appLang}{ok}, -command=>[\&doSplitRecording, \$splitAt, $path, $nMarks])->pack(-side=>'left');
		$sf->Button(-text=>$appText{$appLang}{cancel}, -command=>sub {$w->withdraw; $main::splitRecordingWindow=0;})->pack(-side=>'right');
		}
}

sub doSplitRecording {
	die('Oups, ['.$v2icfg::cmdsUsed{genindex}.'] missing!'."\n") if(not -x $v2icfg::cmdsUsed{genindex});
	my $splitAt = shift;		  
	my $path = shift;
	my $marks = shift;

	$path =~ s/\/$//;
	my $pcmRef = readIdx($path);
	my($total, $off, $delta, $filno) = bp2offset($path, $$splitAt);
	my($mIdx, $mOff, $mFilno, $startFrame, $mType) = findMark($pcmRef, $off, $filno);
	my $dIdx = $startFrame;
	my $pref = dirname($path);
	my $ts   = basename($path);
	my $offset = $mOff+$delta;

	print('found Splitpoint at ('.$mIdx.', '.($offset).")\n") if $main::debug;
	if($ts =~ /^(\d+-\d+-\d+\.\d+)\.(\d+)\.(.+)/) {
		$nts = $1.'.'.($2+5).'.'.$3;	  
		}
	else {
		die('invalid timestamp format! ?!?'."\n");	  
		}
	my $dest = $pref.'/'.$nts;
	my $i = 1;
	my $oMarks = {};
	my $nMarks = {};
	my $cMarks = $#{$marks->{base}};	

	$oMarks->{name} = $marks->{name};
	$oMarks->{base} = ();
	$oMarks->{idx}  = ();
	$nMarks->{name} = $dest.'/'.basename($marks->{name});
	$nMarks->{base} = ();
	$nMarks->{idx}  = ();
	for my $i (0 .. $cMarks){
		push(@{$oMarks->{idx}} , $marks->{idx}[$i])  if $marks->{base}[$i] <= $offset;	  
		push(@{$oMarks->{base}}, $marks->{base}[$i]) if $marks->{base}[$i] <= $offset;	  
		push(@{$nMarks->{idx}} , $marks->{idx}[$i])  if $marks->{base}[$i] >  $offset;	  
		push(@{$nMarks->{base}}, $marks->{base}[$i]) if $marks->{base}[$i] >  $offset;	  
		}
	push(@{$oMarks->{idx}} , $mIdx);
	push(@{$oMarks->{base}}, $mOff+$delta); 
	unshift(@{$nMarks->{idx}}, $mIdx);
	unshift(@{$nMarks->{base}}, $mOff+$delta);

	system('mkdir \''.$dest.'\' 2>/dev/null');
	$cmd = '';
	$cmd = 'cp \''.$path.'/summary.vdr\' \''.$dest.'/summary.vdr\'' if(-s $path.'/summary.vdr');
	$cmd = 'cp \''.$path.'/info.vdr\' \''.$dest.'/info.vdr\'' if(-s $path.'/info.vdr');
	print('command: <|'.$cmd."|>\n") if("$cmd");
	system($cmd) if "$cmd";

	$fn = sprintf('%03d.vdr', $mFilno);	  
	$cmd = 'cp \''.$path.'/'.$fn.'\' \''.$dest.sprintf('/%03d.vdr', $i++).'\'';
	print('command: <|'.$cmd."|>\n");
	system($cmd);

	my $j = $mFilno +1;
	for($fn = sprintf('%03d.vdr', $j); -s $path.'/'.$fn; $j++, $i++, $fn = sprintf('%03d.vdr', $j)){
		$cmd = 'mv \''.$path.'/'.$fn.'\' \''.$dest.sprintf('/%03d.vdr', $i).'\'';
		print('command: <|'.$cmd."|>\n");
		system($cmd);
		}
	$cmd = 'rm -f \''.$path.'/index.vdr\'';
	print('command: <|'.$cmd."|>\n");
	system($cmd);

	$cmd = 'cd \''.$path.'\' && '.$v2icfg::cmdsUsed{genindex};
	print('command: <|'.$cmd."|>\n");
	system($cmd);

	$cmd = 'cd \''.$dest.'\' && '.$v2icfg::cmdsUsed{genindex};
	print('command: <|'.$cmd."|>\n");
	system($cmd);

	corrMarks($nMarks, $dIdx, $dest);
	writeMarks($oMarks->{name}, $oMarks, $pcmRef);
	writeMarks($nMarks->{name}, $nMarks, $pcmRef);
	$main::splitRecordingWindow->withdraw;
}

sub showSplitValue {
	my $lbl = shift;
	my $value = shift;

	$$lbl->configure(-text=>'split at: '.sprintf("%13s", localizeNumber($value)));
}

sub readIdx {
	my $path = shift;
	my $buffer = '';
	my $bytesRead = 0;
	my $fp;
	my @pcm = ();
	my $debug = 0;

	$path .= '/' if not $path =~ /\/$/;
	open($fp, $path.'index.vdr') or die('Oups, could not open index file!'."\n");
	$bytesRead = sysread($fp, $buffer, -s $path.'index.vdr', 0);
	die('Oups, something went wrong on reading index: '.$!."\n") if($bytesRead != -s $path.'index.vdr');
	close($fp);		  

	my $mx = $bytesRead / 8;
	for(my $i=0; $i < $mx; $i++){
		my($off,$type,$mx,undef) = unpack('LCCS', substr($buffer, $i*8, 8));
	   push(@pcm, {offset=>$off, type=>$type, file=>$mx});
		}
	return \@pcm;
}

sub recLength {
	my $path = shift;		  
	my $total = 0;

	$path .= '/' if not $path =~ /\/$/;
	for my $i (1..999){
		my $file = $path.sprintf('%03d.vdr', $i);
		last if not -s $file;
		$total  += -s $file;
		}
	return $total;
}

sub getCmType {
	my $fnMarks = shift;
	my $type    = 'vdrsync';

	if($fnMarks =~ /\.([^.]+)$/){
		my $xt = $1;
		$type = 'projectX' if $xt =~ /pjx/i;
		}
	return $type;	
}

sub readMarks {
	my $path = shift;
	my $fnUser = shift;
	my $pcmRef = shift;
	my $marks = {};
	my $fnMarks;
	my $fp;

	$marks->{base} = ();
	$marks->{name} = '';
	$marks->{idx}  = ();
	$pcmRef = readIdx($path) if not $pcmRef;
	$path .= '/' if not $path =~ /\/$/;
	$fnMarks = $path.$fnUser if("$fnUser" and not $fnMarks =~ /\//);
	$fnMarks = $fnUser if("$fnUser" and $fnMarks =~ /\//);

	if(not "$fnUser" or not -s $fnMarks){
		undef($fnMarks);
		$fnMarks = $path.'marks.pjx' if(-s $path.'marks.pjx');
		$fnMarks = $path.'marks.vdr' if(not $fnMarks and -s $path.'marks.vdr');
	}

	if($fnMarks and open($fp, $fnMarks)){
		$marks->{'name'} = $fnMarks;	  
		print('processing markfile <'.basename($fnMarks).">\n");	  
		while(my $line = <$fp>){
			if($line =~ /^(\d+:\d+:\d+\.\d+)/){
				my($offS, $offT, $idx, undef) = tc2offset($1, $path, $pcmRef);	  

				push(@{$marks->{'idx'}}, $idx);
				push(@{$marks->{'base'}}, $offT);
				}	  
			elsif($line =~ /^(\d+)/){
				my($offS, $offT, $idx, undef) = bp2idx($1, $path, $pcmRef);	  

				push(@{$marks->{'idx'}}, $idx);
				push(@{$marks->{'base'}}, $offT);
				}
			}
		close($fp);
		}
	if(not $marks->{base}){
		print('Oups, no marks found. Create fake ones ...'."\n");	  
		push(@{$marks->{'idx'}}, 0);
		push(@{$marks->{'base'}}, 0);
		my($offS, $offT, $idx, undef) = bp2idx(recLength($path), $path, $pcmRef);	  

		push(@{$marks->{'idx'}}, $idx);
		push(@{$marks->{'base'}}, $offT);
		}
	elsif($#{$marks->{base}} < 1){
		print('Oups, only one mark found. Gonna terminate recording ...'."\n");	  
		my($offS, $offT, $idx, undef) = bp2idx(recLength($path), $path, $pcmRef);	  

		push(@{$marks->{'idx'}}, $idx);
		push(@{$marks->{'base'}}, $offT);
		}
	return $marks;
}

sub writeMarks {
	my $fnMarks = shift;
	my $marks   = shift;
	my $pcmRef  = shift;
	my $type    = getCmType($fnMarks);
	my $fp;
	
	$pcmRef = readIdx(dirname($marks->{file})) if not $pcmRef;
	open($fp, '> '.$fnMarks) or die('could not create new marksfile ['.$fnMarks."]\n");
	print($fp 'CollectionPanel.CutMode=0'."\n") if $type eq 'projectX';
	if($type eq 'projectX'){
		foreach my $mark (@{$marks->{base}}){
			print($fp $mark."\n");	  
			}
		}
	else{
		foreach my $mark (@{$marks->{idx}}){
			print($fp idx2tc($mark)."\n");	  
			}
		}
	close($fp);
}

sub findMark {
	my $pcmRef = shift;
	my $off = shift;
	my $filno = shift;
	my $startFrame = 0;
	my $oldOff = 0;
	my $i = 0;
	my %rv;

	foreach my $cm (@$pcmRef){
		$i++;
		next if($cm->{file} < $filno);
		$startFrame = $i-1 if not $startFrame;
		if($cm->{offset} > $off){
			return ($i-1, $oldOff, $filno, $startFrame, $cm->{type});
			}
		$oldOff = $cm->{offset};
		}
	return ($#{$pcmRef}, $oldOff, $pcmRef->[$#{$pcmRef}]{file}, $startFrame, $pcmRef->[$#{$pcmRef}]{type});
}

sub bp2offset {
	my $path   = shift;
	my $offset = shift;
	my $total  = 0;
	my $oldSum = 0;
	my $filno  = 0;
	my $nDelta = 0;
	my $nOffset= 0;

	$path .= '/' if not $path =~ /\/$/;
	for my $i (1..999){
		my $file = $path.sprintf('%03d.vdr', $i);

		$total += -s $file;
		if(not -s $file){
			if(not $filno and $total <= $offset){
				$filno = $i-1;	  
				$nDelta  = $oldSum - (-s $path.sprintf('%03d.vdr', $filno));		  
				$nOffset = $offset - $nDelta;
				$filno = $i-1;
				}
			last;	  
			}
	   if($total > $offset and $oldSum <= $offset){
			$nDelta  = $oldSum;		  
			$nOffset = $offset - $nDelta;
			$filno = $i;
			}
		$oldSum = $total;
		}
	return ($total, $nOffset, $nDelta, $filno);
}

sub bp2idx {
	my $bp = shift;
	my $path = shift;
	my $pcmRef = shift;
	my(undef, $off, $delta, $filno) = bp2offset($path, $bp);
	my($mIdx, $mOff, $mFilno, $startFrame, $mType) = findMark($pcmRef, $off, $filno);

	return ($mOff, $mOff+$delta, $mIdx-1, $mFilno);
}

sub tc2idx {
	my $tc = shift;		  
	my $frame = 0;
	my $hour = 0;
	my $min = 0;
	my $sec = 0;

	if($tc =~ /(\d+):(\d+):(\d+)\.(\d+)/){
		$hour = $1;
		$min  = $2;
		$sec  = $3;
		$frame= $4;
		}
	return int(($hour * 3600 + $min * 60 + $sec) * 25 + $frame -1);
}

sub idx2offset {
	my $path   = shift;
	my $idx    = shift;
	my $pcmRef = shift;
	my $total  = 0;
	my $offset = $pcmRef->[$idx]{offset};

	$path .= '/' if not $path =~ /\/$/;
	for my $i (1..999){
		my $file = $path.sprintf('%03d.vdr', $i);
		last if not -s $file;
		$total  += -s $file;
		$offset += -s $file if $i < $pcmRef->[$idx]{file};
		}
	return ($pcmRef->[$idx]{offset}, $offset, $idx, $pcmRef->[$idx]{file});		  
}

sub tc2offset {
	my $tc = shift;
	my $path = shift;
	my $pcmRef = shift;
	my $idx = tc2idx($tc);
	my($offS, $offT, undef, $filno) = idx2offset($path, $idx, $pcmRef);

	return ($offS, $offT, $idx, $filno);
}

sub idx2tc {
	my $idx = shift;		  
	my $frame  = int($idx % 25) +1;
	my $tm     = int($idx / 25);
	my $hour   = int($tm / 3600);
	my $min    = int(($tm - 3600 * $hour) / 60);
	my $sec	  = $tm - 3600 * $hour - 60 * $min;

	return sprintf('%u:%02u:%02u.%02u', $hour, $min, $sec, $frame);
}

sub corrMarks {
	my $marks = shift;
	my $corr = shift;
	my $path = shift;
	my $pcmRef = shift;
	my $cMarks = $#{$marks->{base}};

	$path .= '/' if not $path =~ /\/$/;
	$pcmRef = readIdx($path) if not $pcmRef;
	for my $i (0 .. $cMarks){
		$marks->{idx}[$i] -= $corr;
		my(undef, $offT, undef, undef) = idx2offset($path, $marks->{idx}[$i], $pcmRef);

		$marks->{base}[$i] = $offT;
		}
	return $marks;
}

sub startProjectX {
	my $path = '';
	if(scalar(@_)){
		my $listboxes = shift;		  
		my @selection = ();	  
		foreach (qw/aspect audio size ts name prefix path/){
			@selection = $listboxes->{$_}->curselection();
			last if(scalar(@selection));
			}
		return if(not scalar(@selection));	  
	   $path = $listboxes->{path}->get($selection[0]);
		}
	else{
		$path = $config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]->{path};
		}
	my $cmd = $main::nice.' '.$main::cmdsUsed{'java'}.' -jar '.$main::cmdsUsed{'projectX'}.' -out /tmp/jobedit -gui ';
	my $mfn = $path.'marks.cnv.pjx';

	if(not -s $path.'marks.pjx' and -s $path.'marks.vdr'){
		print('gonna convert VDR-marks to ProjectX-Marks (see marks.cnv.pjx). Please be patient...'."\n");
		my $fp;	  
		my $pcmRef = readIdx($path);
		my $marks  = readMarks($path, undef, $pcmRef);
		writeMarks($mfn, $marks, $pcmRef);
		}
	elsif(-s $path.'marks.pjx'){
		$mfn = $path.'marks.pjx';	  
		}
	$cmd .= ' -cut '.$mfn;

	for my $n (1 .. 999){
		my $file = sprintf("%03d.vdr", $n);	  
		last if(not -s $path.$file);
		$cmd .= ' '.$path.$file;
		}

	print("\n$cmd\n\n");
	system($cmd);
}

sub editSummary {
	if($main::summaryWindow and Exists($main::summaryWindow)){
		$main::summaryWindow->deiconify();
		$main::summaryWindow->raise();
		}
	else{
		my $w = $mw->Toplevel();	  

		$w->title($appText{$appLang}{summary});
		$main::summaryWindow = $w;

		my $sf = $w->Frame(-relief=>'groove')->pack(-side=>'bottom');
		my $edit = $w->Scrolled('Text', -scrollbars=>'ose', -width=>50, -height=>12)->pack(-side=>'left');
		$edit->configure(-wrap=>'word');
		$edit->insert('end', $config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]->{summary});

		$sf->Button(-text=>$appText{$appLang}{ok}, -command=>[\&saveSummary, $edit])->pack(-side=>'left');
		$sf->Button(-text=>$appText{$appLang}{'preview'}, -activebackground=>'green', -command=>[\&xPreview, $edit])->pack(-side=>'right', -padx=>50);
		$sf->Button(-text=>$appText{$appLang}{cancel}, -command=>sub {$w->withdraw; $main::summaryWindow=0;})->pack(-side=>'right');
		}
}

sub saveSummary {
	my $edit = shift;

	$config->{'disks'}[$main::currentDisk]{'recordlist'}{'recording'}[$main::currentRecording]->{summary} = $edit->get('1.0', 'end');
	$main::summaryWindow->withdraw();
	$main::summaryWindow = 0;
}

sub isDefault {
	my $disk = shift;		  
	my $key  = shift;
	my $rv   = 1;

	foreach my $skey (keys(%{$$disk->{'settings'}{'item'}{$key}})){
		$rv = 0 if(not $$disk->{'settings'}{'item'}{$key}{$skey} eq $config->{'defaultvalues'}{'item'}{$key}{$skey});
		}
	return $rv;
}

@main::colors = ();
my $fnList = shift;
my $check  = shift;
$main::cacheFn = '.je.recordings.cache';
$main::combat = 0;
$main::combat = 1 if($check eq '--compat' or $fnList eq '--compat');
$fnList = $check if($fnList eq '--compat');
die('please gimme a jobfile to edit...'."\n") if not "$fnList";
our $config = initialize($fnList);
my $defaultValue = $config->{'defaultvalues'}{'item'};
my $jobs = $config->{'disks'};
$main::testOnly    = 1;
$main::currentDisk = 0;
$main::currentRecording = 0;

checkJob('/tmp/jobedit');
$main::mw = MainWindow->new;
$main::mw->title($fnList.' - Job-Editor');
$main::mw->geometry('850x600+0+0');
my $fTop    = $main::mw->Frame(-relief=>'sunken', -bd=>5, -label=>'* * * '.$appText{$appLang}{globals}.' * * *')->pack(-side=>'top', -fill=>'both');
createDefaults($fTop);

my $fBottom = $main::mw->Frame(-relief=>'flat', -borderwidth=>3)->pack(-side=>'bottom', -fill=>'x');
$fBottom->Button(-text=>$appText{$appLang}{'exit'}, -activebackground=>'red', -command=>sub{exit;})->pack(-side=>'left', -padx=>10, -pady=>3);
$fBottom->Button(-text=>$appText{$appLang}{'preview'}, -activebackground=>'green', -command=>\&preview)->pack(-side=>'left', -padx=>50);
$fBottom->Button(-text=>$appText{$appLang}{'newdisk'}, -command=>\&addDisk)->pack(-side=>'left');
$fBottom->Button(-text=>$appText{$appLang}{'description'}, -activebackground=>'#97AAFF', -command=>\&editSummary)->pack(-side=>'right', -padx=>10);
$fBottom->Button(-text=>$appText{$appLang}{'save'}, -activebackground=>'yellow', -command=>[\&saveJob, $fnList])->pack(-side=>'right');
$fBottom->Button(-text=>$appText{$appLang}{'newrec'}, -command=>\&addRecording)->pack(-side=>'right', -padx=>10);

my $fLeft   = $main::mw->Frame(-relief=>'flat', -borderwidth=>3, -label=>$appText{$appLang}{'disks'})->pack(-side=>'left', -fill=>'both');
$main::lbDisks = $fLeft->Scrolled('Listbox', -width=>28, -scrollbars=>'osoe')->pack(-side=>'left', -fill=>'both');
insertDisks($main::lbDisks, $jobs);
$main::lbDisks->bind('<Button-1>', \&changeDiskInfo);
$main::lbDisks->selectionSet(0);

$main::fRight  = $main::mw->Frame(-relief=>'flat', -bd=>1, -label=>'Disk-Config')->pack(-side=>'right', -fill=>'both', -expand=>1);
setupDiskInfo();
setupDiskInfo();
MainLoop;
