#!/usr/bin/perl
#
# This script is a try of doing the job of the burn-plugin in a batch job.
# 		the burn plugin is written by: lordjaxom at vdrportal dot de
# 		this script is written by ...: geronimo at vdrportal dot de
#
# 
# COPYRIGHT:	(c) 2005 Creative Commons by-nc-sa
#					some rights reserved. For details on copyright see:
#					http://creativecommons.org/licenses/by-nc-sa
#
# DISCLAIMER:	NO WARANTY AT ALL. Use at your own risk.
#
# vi: ts=3
#		
# INSTALLATION:
# check config file v2icfg.pm for the apropiate settings for your system
# and put that file into /etc/perl.
# 
# DESCRIPTION: see README on a detailled description
#
# HISTORY:
# 15.01.2006 - chapter-processing now works using a patched vdrsync
#
# 14.01.2006 - check for diskspace
#
# 07.01.2006 - patch for projectX to use marks.vdr for cutting
#
# 01.01.2006 - Support for info.vdr
#
# 01.01.2006 - XMLSimple ersetzt
#
# 25.12.2005 - completely rewritten to use GD instead of Imagemagick 
# 					jobfile format changed to xml.
#
# 20.12.2005 - support for letterbox-menues (aspect: lb)				   
#
# 19.12.2005 - support for languange preference				   
#
# 18.12.2005 - projectX added as possible demuxer.	
# 					added pipe-support, so the jobfile can be passed via stdin.
#
# 16.12.2005 - added checks for used tools and also checks on background images				   
# 				   
# 15.12.2005 - feedback on jobfile-parsing added				   
# 				   
# 14.12.2005 - first success on recordings with AC3 audio streams
#
# 13.12.2005 - first try				   
# 
use Data::Dumper;
use File::Basename;
use GD;
use strict;
use V2I::dvdMenu;
use V2I::jobList;
use V2I::colorSelection;
use v2icfg;

$main::testOnly = 1;
my $doCutting = 1;
my $doMuxing  = 1;
my $doMenues  = 1;
my $doAuthor  = 1;
my $doMkISO   = 1;
my $doCleanup = 0;

##
## code section 
##

sub usage {
	print('vdr2iso.pl is a script to convert vdr recordings to DVD-iso-images.'."\n");
	print('default-values are read from config.pm and may be changed by editing'."\n");
	print('that file or by giving the propiate commandline-option.'."\n\n");
	print('IMPORTANT: No size-checks or size-changes are applied to the recordings, so ');
	print('you have to check for the sizes by your own.'."\n\n");
	print('These options are known:'."\n");
	print(' --help                 - prints this help out.'."\n");
	print(' --pipe                 - use stdin as jobfile. See README about format of the jobfile.'."\n");
	print(' --job=<filename>       - use file as jobfile.'."\n");
	print(' --work=<dirname>       - put temporary fils in given directory.'."\n");
	print(' --iso=<dirname>        - create iso in that directory.'."\n");
	print(' --keep                 - don\'t remove temporary files.'."\n");
	print(' --nice=<level>         - change nice level to given value.'."\n");
	print(' --demuxonly            - process only cut- and demux-steps.'."\n");
	print(' --menuonly             - don\'t convert, just create menu-images.'."\n");
}

sub initialize {
	my $fn = shift;	  
	my $rv;
	my $fp;

	$rv = parseJobfile($fn) if(-s $fn);
	print(Dumper($rv)) if $debug;
	die('invalid jobfile ['.$fn.']') if not "$rv";

	if(open($fp, $main::fnColors)){
		print('found color-list and gonna process it ...'."\n") if $main::debug;	  
		while(my $line = <$fp>){
			if($line =~ /(\d+) (\d+) (\d+)\s+(.+)/){
				my $red   = $1;
				my $green = $2;
				my $blue  = $3;
				my $name  = uc($4);

				print('save color <'.$name.'> as ('.$red.' / '.$green.' / '.$blue.")\n") if $main::debug;
				$colorTable{byName}{$name} = [($red, $green, $blue)];
				$colorTable{byValue}{hexColor([$red, $green, $blue])} = $name;
				push(@main::colors, $name);
				}
			}
		close($fp);	  
		}
	return $rv;
}

sub prepare {
	my $job  = shift;
	my $work = shift;
	my	$settings = $job->{'settings'}->{'item'};
	
	for my $i (0 .. $#{$job->{'recordlist'}->{'recording'}}){
		my($prefix,$name,$ts) = parseRecording($job->{'recordlist'}->{'recording'}[$i]->{'path'});	  

		my $dest = $work.'/VDRSYNC.'.$i;
		my $cmd  = 'mkdir -p \''.$dest.'\'';
		print('command: <|'.$cmd."|>\n") if $main::debug;
		system($cmd);					  

		$job->{'recordlist'}->{'recording'}[$i]->{'path'}   .= '/' if not $job->{'recordlist'}->{'recording'}[$i]->{'path'} =~ /\/$/;  
		$job->{'recordlist'}->{'recording'}[$i]->{'title'}   = $name                             if not exists($job->{'recordlist'}->{'recording'}[$i]->{'title'});  
		$job->{'recordlist'}->{'recording'}[$i]->{'chapLen'} = $settings->{'chapLen'}->{'value'} if not exists($job->{'recordlist'}->{'recording'}[$i]->{'chapLen'});
		$job->{'recordlist'}->{'recording'}[$i]->{'demux'}   = $settings->{'demux'}->{'value'}   if not exists($job->{'recordlist'}->{'recording'}[$i]->{'demux'});
		$job->{'recordlist'}->{'recording'}[$i]->{'lang'}    = $settings->{'language'}->{'value'}if not exists($job->{'recordlist'}->{'recording'}[$i]->{'lang'});
		$job->{'recordlist'}->{'recording'}[$i]->{'ts'}      = substr($ts,0,16);
		$job->{'recordlist'}->{'recording'}[$i]->{'prefix'}  = $prefix;

		if(not "$job->{'recordlist'}->{'recording'}[$i]->{'summary'}"){
			my $summary = $job->{'recordlist'}->{'recording'}[$i]->{'title'};
			my $fpSum;
			if(-s $job->{'recordlist'}->{'recording'}[$i]->{'path'}.'summary.vdr'){
				if(open($fpSum, $job->{'recordlist'}->{'recording'}[$i]->{'path'}.'summary.vdr')){
					$summary = '';	  
					while(my $line = <$fpSum>){
						$summary .= $line;	  
						}
					close($fpSum);
					}
				$job->{'recordlist'}->{'recording'}[$i]->{'summary'} = $summary;
				}
			elsif(-s $job->{'recordlist'}->{'recording'}[$i]->{'path'}.'info.vdr'){
				if(open($fpSum, $job->{'recordlist'}->{'recording'}[$i]->{'path'}.'info.vdr')){
					$summary = '';
					while(my $line = <$fpSum>){
						if($line =~ /^T (.+)$/){
							$job->{'recordlist'}->{'recording'}[$i]->{'title'} = $1;
							}
						elsif($line =~ /^D (.+)$/){
							$job->{'recordlist'}->{'recording'}[$i]->{'summary'} = $1;
							}
						}
					close($fpSum);
					}
				}
			}
		}
}

sub getCmType {
	my $fnMarks = shift;
	my $type    = 'vdrsync';

	if($fnMarks =~ /\.([^.]+)$/){
		my $xt = $1;
		$type = 'projectX' if $xt =~ /pjx/i;
		}
	return $type;	
}

sub cutAndDemux {
	my $job = shift;
	my $work = shift;
	my $i = 0;
	my %cmds;
	$cmds{'projectX'} = $main::nice.' '.$main::cmdsUsed{'java'}.' -jar '.$main::cmdsUsed{'projectX'}.' -out ';
	$cmds{'vdrsync'}  = $main::nice.' '.$main::cmdsUsed{'vdrsync'}.' -cut -o '; 

	foreach my $rec (@{$job->{'recordlist'}->{'recording'}}){
		my $path = $rec->{'path'};	
		my $dest = $work.'/VDRSYNC.'.$i;

		print("\n".'------------------------------------------------------------------------------'."\n");
		print('--- Cut and demux recording ['.$rec->{'title'}.']...'."\n");
		print('------------------------------------------------------------------------------'."\n");
		my $cmd = $cmds{$rec->{'demux'}};	
		die('unsupported demuxer: <'.Dumper($rec).">\n") if not "$cmd";
		$cmd .= ' \''.$dest.'\' ';
		
		if($rec->{'demux'} =~ /projectX/){
			my $fnCutMarks = '';	  
			if("$rec->{'cutMarks'}"){
				$fnCutMarks = $rec->{'cutMarks'} if(-s $rec->{'cutMarks'});
				$fnCutMarks = $path.$rec->{'cutMarks'} if(-s $path.$rec->{'cutMarks'});
				die('given cutmark<'.$rec->{'cutMarks'}.'> NOT found and should ABORT.'."\n") if(not -s $fnCutMarks and $abortOnCutmarkNotFound);
				}
			if(-s $fnCutMarks) { print('cutmarks-file found. Gonna use '.$fnCutMarks."\n"); }
	  	   elsif(-s $path.'001.vdr[0].Xcl'){ $fnCutMarks = $path.'001.vdr[0].Xcl'; }
			elsif(-s $path.'marks.pjx')  { $fnCutMarks = $path.'marks.pjx'; }
			elsif(-s $path.'marks.vdr')  { 
				my $fp;
				my $cutMarks = 'CollectionPanel.CutMode=4'."\n";
				if(open($fp, $path.'marks.vdr')){
					while(my $line = <$fp>){
						$cutMarks .= $line;	  
						}
					close($fp);
					}
				if(open($fp, '> '.$path.'marks.vdr.pjx')){
					print($fp $cutMarks);	  
					close($fp);	  
		   		$fnCutMarks = $path.'marks.vdr.pjx '; 
					}
			}
			$cmd .= ' -cut '.$fnCutMarks.' ' if "$fnCutMarks";
			for my $n (1 .. 999){
				my $file = sprintf("%03d.vdr", $n);	  
				last if(not -s $path.$file);
				$cmd .= ' '.$path.$file;
				}
			}
		elsif($rec->{'demux'} =~ /vdrsync/){
			my $fnCutMarks = '';

			if("$rec->{'cutMarks'}"){
				$fnCutMarks = $rec->{'cutMarks'} if(-s $rec->{'cutMarks'});
				$fnCutMarks = $path.$rec->{'cutMarks'} if(-s $path.$rec->{'cutMarks'});
				die('given cutmark<'.$rec->{'cutMarks'}.'> NOT found and should ABORT.'."\n") if(not -s $fnCutMarks and $abortOnCutmarkNotFound);
			}
			if(-s $fnCutMarks) { print('cutmarks-file found. Gonna use '.$fnCutMarks."\n"); }
			if(getCmType($fnCutMarks) eq 'projectX'){
				print('Oups, given projectX-cutmarks and active demuxer is vdrsync - have 2 convert cutmarks...'."\n");
				my $fp;	  
				my $pcmRef = readIdx($path);
				my $marks  = readMarks($path, $fnCutMarks, $pcmRef);

				$fnCutMarks =~ s/\.pjx$/\.vdr/i;
				writeMarks($fnCutMarks, $marks, $pcmRef);
			}
			$cmd .= ' -user-cutmarks=\''.$fnCutMarks.'\' ' if "$fnCutMarks" and -s $fnCutMarks;
			$cmd .= ' \''.$path.'\' ';	  
			}
		print('command: <|'.$cmd."|>\n");
		system($cmd) if not $main::testOnly;
		$i++;
		}
	print("\n");
}

sub muxRecordings {
	my $job = shift;
	my $work = shift;
	my $i=0;

	foreach my $rec (@{$job->{'recordlist'}->{'recording'}}){
		my $dest = $work.'/VDRSYNC.'.$i;

		print("\n".'------------------------------------------------------------------------------'."\n");
		print('--- mux recording ['.$rec->{'title'}.']...'."\n");
		print('------------------------------------------------------------------------------'."\n");
		my $cmd  = $main::nice.' '.$main::cmdsUsed{mplex}.' -f 8 -vbr -v 1 -o \''.$dest.'/movie.mpg\' ';
		if($rec->{'demux'} =~ /projectX/){
			$cmd .= '\''.$dest.'/001.m2v\' ';
			if(-s $dest.'/001['.$rec->{'lang'}.'].ac3'){
				$cmd .= '\''.$dest.'/001['.$rec->{'lang'}.'].ac3\' ';
				}		  
			elsif(-s $dest.'/001.ac3'){
				$cmd .= '\''.$dest.'/001.ac3\' ';
				}
			else{
				$cmd .= '\''.$dest.'/001.mp2\' ';
				}
			}
		else{
			$cmd .= '\''.$dest.'/vdrsync.mpv\' ';
			if(-s $dest.'/vdrsync.ac3'){
				$cmd .= '\''.$dest.'/vdrsync.ac3\' ';	  
				}
			else{
				$cmd .= '\''.$dest.'/vdrsync0.mpa\' ';	  
				}
			}
		print("command: <|$cmd|>\n");
		system($cmd) if not $main::testOnly;
		$i++;
		}
	print("\n");
}

sub createChapterString {
	my $chapLength = shift;	  
	my $step = 0;
	my $rv = '0';
	my $i = 1;
	my $h = 0;
	my $m = 0;
	$chapLength = 5 if($chapLength < 1 or $chapLength > 10);
	while($step < 240){
		$step = $i++ * $chapLength;	  
		$h = sprintf("%d", $step / 60);
		$m = sprintf("%02d", $step % 60);
		$rv .= ','.$m.':00' if $h lt 1;
		$rv .= ','.$h.':'.$m.':00' if $h ge 1;
		}
	return $rv;
}

sub enufSpace {
	my $work = shift;
	my $cmd = 'df \''.$work.'\' \''.$iso.'\' 2>/dev/null';
	my @needed = (15000000, 5000000);
	my $fp;
	my $rv = 1;

	open($fp, $cmd.' |') or die('could not determine diskspace '.$!."\n");
	my $check = '';
	my $i = 0;
	while(my $line = <$fp>){
		next if $line =~ /^Filesystem/;
		$check .= $line;
		if($check =~ /([^\s]+)\s+(\d+)\s+(\d+)\s+(\d+)/){
			my $device = $1;
			my $total  = $2;
			my $used   = $3;
			my $free   = $4;

			print($device.' has size='.$total.', used='.$used.' and free='.$free."\n") if $main::debug;
			$rv = 0 if($free < $needed[$i]);
			$check = '';
			$i++;
			}
		else{
			print('Oups, pattern did not match?!?'."\n");
			$rv = 0;
			}
		}
	close($fp);
	return $rv;
}

sub readIdx {
	my $path = shift;
	my $buffer = '';
	my $bytesRead = 0;
	my $fp;
	my @pcm = ();
	my $debug = 0;

	$path .= '/' if not $path =~ /\/$/;
	open($fp, $path.'index.vdr') or die('Oups, could not open index file!'."\n");
	$bytesRead = sysread($fp, $buffer, -s $path.'index.vdr', 0);
	die('Oups, something went wrong on reading index: '.$!."\n") if($bytesRead != -s $path.'index.vdr');
	close($fp);		  

	my $mx = $bytesRead / 8;
	for(my $i=0; $i < $mx; $i++){
		my($off,$type,$mx,undef) = unpack('LCCS', substr($buffer, $i*8, 8));
	   push(@pcm, {offset=>$off, type=>$type, file=>$mx});
		}
	return \@pcm;
}

sub recLength {
	my $path = shift;		  
	my $total = 0;

	$path .= '/' if not $path =~ /\/$/;
	for my $i (1..999){
		my $file = $path.sprintf('%03d.vdr', $i);
		last if not -s $file;
		$total  += -s $file;
		}
	return $total;
}

sub getCmType {
	my $fnMarks = shift;
	my $type    = 'vdrsync';

	if($fnMarks =~ /\.([^.]+)$/){
		my $xt = $1;
		$type = 'projectX' if $xt =~ /pjx/i;
		}
	return $type;	
}

sub readMarks {
	my $path = shift;
	my $fnUser = shift;
	my $pcmRef = shift;
	my $marks = {};
	my $fnMarks;
	my $fp;

	$marks->{base} = ();
	$marks->{name} = '';
	$marks->{idx}  = ();
	$pcmRef = readIdx($path) if not $pcmRef;
	$path .= '/' if not $path =~ /\/$/;
	$fnMarks = $path.$fnUser if("$fnUser" and not $fnMarks =~ /\//);
	$fnMarks = $fnUser if("$fnUser" and $fnMarks =~ /\//);

	if(not "$fnUser" or not -s $fnMarks){
		undef($fnMarks);
		$fnMarks = $path.'marks.pjx' if(-s $path.'marks.pjx');
		$fnMarks = $path.'marks.vdr' if(not $fnMarks and -s $path.'marks.vdr');
	}

	if($fnMarks and open($fp, $fnMarks)){
		$marks->{'name'} = $fnMarks;	  
		print('processing markfile <'.basename($fnMarks).">\n");	  
		while(my $line = <$fp>){
			if($line =~ /^(\d+:\d+:\d+\.\d+)/){
				my($offS, $offT, $idx, undef) = tc2offset($1, $path, $pcmRef);	  

				push(@{$marks->{'idx'}}, $idx);
				push(@{$marks->{'base'}}, $offT);
				}	  
			elsif($line =~ /^(\d+)/){
				my($offS, $offT, $idx, undef) = bp2idx($1, $path, $pcmRef);	  

				push(@{$marks->{'idx'}}, $idx);
				push(@{$marks->{'base'}}, $offT);
				}
			}
		close($fp);
		}
	if(not $marks->{base}){
		print('Oups, no marks found. Create fake ones ...'."\n");	  
		push(@{$marks->{'idx'}}, 0);
		push(@{$marks->{'base'}}, 0);
		my($offS, $offT, $idx, undef) = bp2idx(recLength($path), $path, $pcmRef);	  

		push(@{$marks->{'idx'}}, $idx);
		push(@{$marks->{'base'}}, $offT);
		}
	elsif($#{$marks->{base}} < 1){
		print('Oups, only one mark found. Gonna terminate recording ...'."\n");	  
		my($offS, $offT, $idx, undef) = bp2idx(recLength($path), $path, $pcmRef);	  

		push(@{$marks->{'idx'}}, $idx);
		push(@{$marks->{'base'}}, $offT);
		}
	return $marks;
}

sub writeMarks {
	my $fnMarks = shift;
	my $marks   = shift;
	my $pcmRef  = shift;
	my $type    = getCmType($fnMarks);
	my $fp;
	
	$pcmRef = readIdx(dirname($marks->{file})) if not $pcmRef;
	open($fp, '> '.$fnMarks) or die('could not create new marksfile.'."\n");
	print($fp 'CollectionPanel.CutMode=0'."\n") if $type eq 'projectX';
	if($type eq 'projectX'){
		foreach my $mark (@{$marks->{base}}){
			print($fp $mark."\n");	  
			}
		}
	else{
		foreach my $mark (@{$marks->{idx}}){
			print($fp idx2tc($mark)."\n");	  
			}
		}
	close($fp);
}

sub findMark {
	my $pcmRef = shift;
	my $off = shift;
	my $filno = shift;
	my $startFrame = 0;
	my $oldOff = 0;
	my $i = 0;
	my %rv;

	foreach my $cm (@$pcmRef){
		$i++;
		next if($cm->{file} < $filno);
		$startFrame = $i-1 if not $startFrame;
		if($cm->{offset} > $off){
			return ($i-1, $oldOff, $filno, $startFrame, $cm->{type});
			}
		$oldOff = $cm->{offset};
		}
	return ($#{$pcmRef}, $oldOff, $pcmRef->[$#{$pcmRef}]{file}, $startFrame, $pcmRef->[$#{$pcmRef}]{type});
}

sub bp2offset {
	my $path   = shift;
	my $offset = shift;
	my $total  = 0;
	my $oldSum = 0;
	my $filno  = 0;
	my $nDelta = 0;
	my $nOffset= 0;

	$path .= '/' if not $path =~ /\/$/;
	for my $i (1..999){
		my $file = $path.sprintf('%03d.vdr', $i);

		$total += -s $file;
		if(not -s $file){
			if(not $filno and $total <= $offset){
				$filno = $i-1;	  
				$nDelta  = $oldSum - (-s $path.sprintf('%03d.vdr', $filno));		  
				$nOffset = $offset - $nDelta;
				$filno = $i-1;
				}
			last;	  
			}
	   if($total > $offset and $oldSum <= $offset){
			$nDelta  = $oldSum;		  
			$nOffset = $offset - $nDelta;
			$filno = $i;
			}
		$oldSum = $total;
		}
	return ($total, $nOffset, $nDelta, $filno);
}

sub bp2idx {
	my $bp = shift;
	my $path = shift;
	my $pcmRef = shift;
	my(undef, $off, $delta, $filno) = bp2offset($path, $bp);
	my($mIdx, $mOff, $mFilno, $startFrame, $mType) = findMark($pcmRef, $off, $filno);

	return ($mOff, $mOff+$delta, $mIdx-1, $mFilno);
}

sub tc2idx {
	my $tc = shift;		  
	my $frame = 0;
	my $hour = 0;
	my $min = 0;
	my $sec = 0;

	if($tc =~ /(\d+):(\d+):(\d+)\.(\d+)/){
		$hour = $1;
		$min  = $2;
		$sec  = $3;
		$frame= $4;
		}
	return int(($hour * 3600 + $min * 60 + $sec) * 25 + $frame -1);
}

sub idx2offset {
	my $path   = shift;
	my $idx    = shift;
	my $pcmRef = shift;
	my $total  = 0;
	my $offset = $pcmRef->[$idx]{offset};

	$path .= '/' if not $path =~ /\/$/;
	for my $i (1..999){
		my $file = $path.sprintf('%03d.vdr', $i);
		last if not -s $file;
		$total  += -s $file;
		$offset += -s $file if $i < $pcmRef->[$idx]{file};
		}
	return ($pcmRef->[$idx]{offset}, $offset, $idx, $pcmRef->[$idx]{file});		  
}

sub tc2offset {
	my $tc = shift;
	my $path = shift;
	my $pcmRef = shift;
	my $idx = tc2idx($tc);
	my($offS, $offT, undef, $filno) = idx2offset($path, $idx, $pcmRef);

	return ($offS, $offT, $idx, $filno);
}

sub idx2tc {
	my $idx = shift;		  
	my $frame  = int($idx % 25) +1;
	my $tm     = int($idx / 25);
	my $hour   = int($tm / 3600);
	my $min    = int(($tm - 3600 * $hour) / 60);
	my $sec	  = $tm - 3600 * $hour - 60 * $min;

	return sprintf('%u:%02u:%02u.%02u', $hour, $min, $sec, $frame);
}

sub localizeTS {
	my $raw = shift;
	my $rv;

	if($raw =~ /^(\d+)-(\d+)-(\d+)\.(\d+)\.(\d+)/){
		if(   $appLang eq 'de') { $rv = $3.'.'.$2.'.'.$1.' '.$4.':'.$5; }
		elsif($appLang eq 'en') { $rv = $2.'/'.$3.'/'.$1.' '.$4.':'.$5; }
		else                    { $rv = $1.'-'.$2.'-'.$3.' '.$4.'.'.$5; }
		}
	else {
		$rv = substr($raw,0,16);			  
		} 
	return $rv;
}

sub authorDisk { 
   my $job = shift;
	my $work = shift;
	my $cmd = 'mkdir \''.$work.'/DVDAUTHOR\'';
	my $i=0;

	print("command: <|$cmd|>\n");
	system($cmd);

	my $jfn = $work.'/dvd.xml';
	my $fpx;
	print("\n".'------------------------------------------------------------------------------'."\n");
	print('--- author dvd ['.$job->{'title'}.']...'."\n");
	print('------------------------------------------------------------------------------'."\n");
	if(open($fpx, ">$jfn")){
		print($fpx '<dvdauthor dest="'.$work.'/DVDAUTHOR">'."\n");
		print($fpx '  <vmgm>'."\n");
		print($fpx '    <menus>'."\n");
		print($fpx '      <pgc entry="title">'."\n");
		print($fpx '        <vob file="'.$work.'/menu-bg.mpg"/>'."\n");
		foreach my $i (0 .. $#{$job->{'recordlist'}->{recording}}){
			print($fpx '	     <button>jump titleset '.($i+1).' menu;</button>'."\n");
			}
		print($fpx '      </pgc>'."\n");
		print($fpx '    </menus>'."\n");
		print($fpx '  </vmgm>'."\n");
		foreach my $i (0 .. $#{$job->{'recordlist'}->{recording}}){	
			my $chapters = createChapterString($job->{'recordlist'}->{'recording'}[$i]->{'chapLen'});
			
			print($fpx '  <titleset>'."\n");
			print($fpx '    <menus>'."\n");
			print($fpx '      <pgc entry="root">'."\n");
			print($fpx '        <vob file="'.$work.'/VDRSYNC.'.$i.'/menu.mpg"/>'."\n");
			print($fpx '        <button>jump title 1 chapter 1;</button>'."\n");
			print($fpx '        <button>jump vmgm menu;</button>'."\n");
			print($fpx '      </pgc>'."\n");
			print($fpx '    </menus>'."\n");
			print($fpx '    <titles>'."\n");
			print($fpx '      <pgc>'."\n");
			print($fpx '        <vob file="'.$work.'/VDRSYNC.'.$i.'/movie.mpg"'."\n");
			print($fpx '        chapters="'.$chapters.'"/>'."\n");
			print($fpx '        <post>call vmgm menu;</post>'."\n");
			print($fpx '      </pgc>'."\n");
			print($fpx '    </titles>'."\n");
			print($fpx '  </titleset>'."\n");
			} 
		print($fpx '</dvdauthor>'."\n");
		close($fpx);
		}	
	$cmd = $main::nice.' '.$main::cmdsUsed{dvdauthor}.' -x \''.$work.'/dvd.xml\'';
	print("command: <|$cmd|>\n");
	system($cmd) if not $main::testOnly;
}

sub createISO {
   my $job  = shift;
	my $work = shift;
	my $ts   = shift;

	print("\n".'------------------------------------------------------------------------------'."\n");
	print('--- create iso ['.$job->{'title'}.']...'."\n");
	print('------------------------------------------------------------------------------'."\n");
	my $cmd = $main::nice.' '.$main::cmdsUsed{mkisofs}.' -V \''.substr($job->{title},0,30)
				.'\' -dvd-video \''.$work.'/DVDAUTHOR\' > \''.$main::iso.'/job-'.$ts.'.iso\' ';
	print("command: <|$cmd|>\n");
	system($cmd) if not $main::testOnly;
}

sub cleanUP {
   my $job  = shift;
	my $work = shift;
	my $ts   = shift;
	my $i=0;

	print("\n".'------------------------------------------------------------------------------'."\n");
	print('--- cleanup ['.$job->{'title'}.']...'."\n");
	print('------------------------------------------------------------------------------'."\n");
	if(-s $main::iso.'/job-'.$ts.'.iso'){
		my $cmd = 'rm -rf \''.$work.'\'';
		print("command: <|$cmd|>\n");
		system($cmd) if not $main::testOnly;
		} 
	else{
		print('Oups, no iso found (should be ['.$main::iso.'/job-'.$ts.'.iso]) - cleanup ABORTED.'."\n");	  
		}	  
}

##
## main
##
my $fnList = 'stdin';
my $cleanRecordings = 0;
$main::menuTest = 0;
while(my $scratch = pop){
	if($scratch =~ /^-+\?/
		or $scratch =~ /^--help/)     { exit(usage()); }
	elsif($scratch =~ /^REALLY$/)    { $main::testOnly = 0; }
	elsif($scratch =~ /^--pipe/)     { $fnList = 'stdin'; }
	elsif($scratch =~ /^--keep/)     { $doCleanup = 0; }
	elsif($scratch =~ /^--demuxonly/ or $scratch =~ /^--demuxOnly/){
		$doCutting = 1;
		$doMuxing  = 0;
		$doMenues  = 0;
		$doAuthor  = 0;
		$doMkISO   = 0;
		$doCleanup = 0;
		}
	elsif($scratch =~ /^--menuonly/ or $scratch =~ /^--menuOnly/){
		$main::menuTest = 1;
		$doCutting = 0;
		$doMuxing  = 0;
		$doMenues  = 1;
		$doAuthor  = 0;
		$doMkISO   = 0;
		$doCleanup = 0;
		}
	elsif($scratch =~ /^--removeRecordings/)     { $cleanRecordings = 1; }
	elsif($scratch =~ /^--job(?:\:|=)(.+)/)      { $fnList = $1; }
	elsif($scratch =~ /^--work(?:\:|=)(.+)/)     { $main::work = $1; }
	elsif($scratch =~ /^--iso(?:\:|=)(.+)/)      { $main::iso = $1; }
	elsif($scratch =~ /^--nice(?:\:|=)(.+)/)     { $main::niceLevel = $1; }
	}
foreach my $cmd (keys(%main::cmdsUsed)){
	die('Oups, '.$cmd.' not found!'."\n") if(not -s $main::cmdsUsed{$cmd});
	}
$main::combat = 0;
$main::nice = 'nice -n '.$main::niceLevel;
$fnList = '-' if $fnList =~ /^stdin$/;
my $config = initialize($fnList);
my $defaultValues = $config->{'defaultvalues'}{'item'};
my $jobs = $config->{'disks'};

foreach my $job (@$jobs){
	my $tsStart  = time();
	my	$settings = $job->{'settings'}->{'item'};
	next if not exists($job->{'recordlist'}{'recording'});

	$settings->{'imgBack'}   = $defaultValues->{'imgBack'}   if not exists($settings->{'imgBack'});
	$settings->{'colShadow'} = $defaultValues->{'colShadow'} if not exists($settings->{'colShadow'});
	$settings->{'colBack'}   = $defaultValues->{'colBack'}   if not exists($settings->{'colBack'});
	$settings->{'language'}  = $defaultValues->{'language'}  if not exists($settings->{'language'});
	$settings->{'aspect'}    = $defaultValues->{'aspect'}    if not exists($settings->{'aspect'});
	$settings->{'fntStd'}    = $defaultValues->{'fntStd'}    if not exists($settings->{'fntStd'});
	$settings->{'fntBig'}    = $defaultValues->{'fntBig'}    if not exists($settings->{'fntBig'});
	$settings->{'header'}    = $defaultValues->{'header'}    if not exists($settings->{'header'});
	$settings->{'demux'}     = $defaultValues->{'demux'}     if not exists($settings->{'demux'});
	$settings->{'chapLen'}   = $defaultValues->{'chapLen'}   if not exists($settings->{'chapLen'});
	
	print('############################ START disk PROCESSING ###########################'."\n");
	print('work on disk with title....: '.$job->{title}."\n");	  
	print('          workdir-timestamp: '.$tsStart."\n");	  
	my $scratch = $main::work.'/.v2i.'.$tsStart;

	die("\n".$appText{$appLang}{nospace}."\n\n") if not enufSpace($scratch);
	prepare(      $job, $scratch);
	cutAndDemux(  $job, $scratch) if $doCutting;
	muxRecordings($job, $scratch) if $doMuxing;
	createMenus(  $job, $scratch) if $doMenues;
	authorDisk(   $job, $scratch) if $doAuthor;
	createISO(    $job, $scratch, $tsStart) if $doMkISO;
	cleanUP(      $job, $scratch, $tsStart) if $doCleanup;
	}
